                        zeusemulate "48K"               ;

                        org $8000                       ;

AppStart                ld a,2                          ;
                        call 5633                       ;

Lp                      ld de,sString                   ;
                        ld bc,lString                   ;
                        call 8252                       ;
                        jp Lp                           ;

sString                 db "Hello World "               ;
lString                 equ $-sString                   ;

AppLength               equ *-AppStart                  ;

Zeus_PC                 equ AppStart                    ;

; We need a loading screen for mode 1 files

                        import_bin "Mode3.scr",$4000

; Save this as a mode 3 tzx

                        output_tzx "Mode3.tzx","Mode3","",AppStart,AppLength,3

; Save this as a mode 3 tap

;                        output_tap "Mode3.tap","Mode3","",AppStart,AppLength,3

/* From the History/Documentation

    MODE 3 (Uses a custom loader. 48K, 128K models fully supported)

    Zeus can also automatically generate a "PROGRAM" block containing a tape loader, and add the
    code as a data block after it... this uses a loader like the ones I used to use in the 80's
    that load the screen at the same time as they load the data.

    This mode cannot be used on the 16K Spectrum. On the 128K models it can load to any page or pages.

    This mode can load multiple blocks of memory, which can be on any 16K page, or even overlap pages.
    These blocks can be of any size and start at any address.

    The command is the same as for mode 1, including the start and length of the first block as normal:

    output_tzx "pc filename","tape filename","comment",Start,Length, 3 ,ExecuteAddr
    or it supports some options, mainly you can change the border colours
    output_tzx "pc filename","tape filename","comment",Start,Length, 3 ,ExecuteAddr,Options

    But you can add extra blocks either as Start,Length pairs after the full command like this:

    output_tzx "pc filename","tape filename","comment",Start,Length, g3 ,ExecuteAddr,Options ,Start2,Length2
    and so on, or using "output_tzx_block" commands:

    output_tzx_block "pc filename",Start,Length
    output_tzx_block "pc filename",Start,Length,Start2,Length2,Start3,Length3 and so on...

    Note that the "pc filename" must be the same in the block commands as in the original, that is
    how zeus knows which output file you are adding blocks to.

    The "start" address can be a normal 16-bit $4000..$FFFF address, or anywhere in Zeus's page memory as
    described above using greater than 16-bit addressing. So, for example, for a 128K Spectrum page 0 can
    be addressed at $10000, where that number has four zero's.
    You could use "zeuspage(0)" as well.

    The "length" can also be larger than 64K... for example, saving from $10000 for $10000 would save
    pages 0,1,2, and 3.

    To use this loader sensibly you should have a loading screen loaded in memory at $4000 .. $5AFF
    for it to use... how Zeus chooses to load the screen depends on how much of it contains visible
    pixels; if entire lines of pixels are $00 Zeus will not load them. Also this mode optimises zeros
    at the beginning and ending of lines... Zeus will load non-zero attribute lines when they are
    required, just before any pixels using them are loaded.

    For example, if you have an assembler source for a game that starts at $8000, is $2000 bytes
    long and you have a loading screen for it in a file called "screen.scr" then Zeus can generate
    a tzx file with a nice loading screen using the commands:

    import_bin "screen.scr",$4000 ; This gets the loading screen and puts it in memory in place
    output_tzx "game.tzx","mygame","(c) Etc",$8000,$2000,3,$8000 ; Generate the tzx file
    where the 3 indicates this is the required TZX format.

    To load a program generated using Zeus use LOAD "" on your Spectrum or emulator.

    The "Options" optional parameter allows you to set the colours used while the loader is operating.
    (Same as MODE 1)

    output_tzx "tony","tony","Hideous!",$7F00,$4000,3,$7F00,$00123456

    The most significant byte is reserved for other loader options to be added in future. Set it
    to zero for the moment.

    LIMITATIONS:

    48K Spectrum.

    You can load blocks of code to any address from $4000 to $FFFF, but you cannot load all 48K.
    The loader itself needs to be placed in memory on a page boundary between $8000..$FE00.
    Zeus will handle deciding where to put the loader, but you must leave at least one whole page
    of memory free somewhere for it to do so.

    When your code starts running (at ExecuteAddr) interrupts will be disabled and the stack
    pointer will be left at some address between $8000..$BF00, so I STRONGLY suggest the first
    thing your code does is to load the SP register with a safe value.

    128K Spectrum.

    You can load blocks of code to any address from $4000 to $FFFF, including all 48K. You can
    also load as much as you like to any other block, or blocks, but you cannot quite load all
    128K, you must leave a couple of hundred bytes free for Zeus to use.

    The loader itself needs to be placed in memory on a page boundary between $8000..$BF00 while
    running, but if none of this memory is available (because you are loading all over it) Zeus
    will put the loader somewhere in there and add code so that when the load is finished the loader
    will automatically be overwritten with your code, so you need not be concerned about it.

    However, this means that you must leave about 270 bytes free somewhere in the 128K... if you
    do, Zeus will find it and use it automatically.

    The execute address should be between $4000..$BFFF... if it is above $C000 the code may not
    be paged in correctly if Zeus needed to find a different page to put the overlay copy of the
    user's code in.

    When your code starts running (at ExecuteAddr) interrupts will be disabled and the stack
    pointer will be left at some address between $8000..$BF00, so I STRONGLY suggest the first
    thing your code does is to load the SP register with a safe value... also, the page register
    (IO address $7FFD) may have been written with a random page, so you should also set this with
    whatever page you need. Well, it's not random, but it will be wherever Zeus chose to leave it.

    NOTE - the loader will also change the ROM page in I/O port $7FFE, so you will need to reset
    this at the start of your code if you intend to call ROM routines. If you haven't loaded over
    the ROM variable area then having this code at the start of your code should put it back:

                        ld a,($5B5C)      ; Previous value of port is stored here
                        ld bc,$7ffd       ; Page register address
                        out (c),a         ; Put it back as the BASIC, etc, expects.

    And as an added bonus for reading this far, bits 26..24 of the "option" word are actually used
    in mode 3... have fun finding out what they do ;)

    Use LOAD "" to load this. Or will load automatically on most emulators.

*/

