/*---------------------------------------------------------------------------------------

 Halls 2011. Spectrum source

 From version 10.10 3/3/1983
      version 11   19/2/2006
      version 12   22/5/2011
      version 18    6/9/2016

Hello. Simon 'ere. These sources are hideous. The formatting is dreadful.
There are reasons...

When Neil, Martin and I started writing this game back in 1982(!) we were using
a development system that was a home-brew computer I'd designed, running an OS
largely stolen from a Nascom and using an assembler that Neil had written.
These were very early days indeed for computer games and microprocessor-based
software development ;)

The development machine had 48K of memory. The editor/assembler (an early version
of Zeus) tokenised the source code in order to save room, and the object code was
transferred directly to the target machine's memory via a fast parallel hardware
interface. This put us well up on other games developers of the era who were often
writing code directly on the target machines themselves... Urgh.

But even so Halls was a large source, so although we started out with meaningful
labels and neat spacing as we got towards the end of the project we were being
seriously hampered by the memory limitation of the development machine. This meant
we had to start doing all sorts of god-awful things to save room in the source.

First off all the long labels became converted to terse, short ones and all of
the comments were removed. That was painful. Then finally all the white-space had
to be shrunk - this was desperate stuff! Zeus was already compressing any number
of multiple spaces to only two bytes in the source, but even so we had so little
room left we had to eliminate any extra space. Finally, any "defb 0" was replaced
by "nop" - this was hideous, but even with Zeus's compression "defb 0" took three
bytes - one for the tokenised "defb", one for the space and one for the "0". But
"nop" was tokenised down to a single byte...

But somehow we managed to write Halls, entirely in assembler, in a small number of
weeks and release a complex game which was very reliable. It was a ground-breaking
game - we (Chris Clark, I think) coined the term "arcade-adventure" to describe it
and one day I will bore everyone by writing about some of the things that happened
when we first showed it to people... But not now, I'm extremely tired. What was that?
Oh. Charming...

Now we move forward 28 years to the days where people start emulating old games on
touch-screen devices, and Halls has a problem in that it has far too many controls.

What? No, nobody ever said that before. Never... Anyway, the wise had reckoned the
number of the keys, and the number of the keys was n-n-n-nineteen. Ideally we now
want five... So, I decided to put a couple of sessions into re-writing it to see
if I could eliminate a few keys, and then went on to add a (slightly) nicer looking
front-end and some options and so on. So now we have a version that plays better
with fewer controls.

What I haven't done is spent much, if any, time tidying up the sources. So what
you see is representative of a very crammed exercise from 1982/3, and nothing like
the sort of source design-design produce under normal circumstances. I felt that
needed to be said. Ah, vanity...

So, most of this was written more than half my life ago, by three people having
fun. And it has been fun to re-visit the code... Enjoy...

And remember - Things are still fundamentally better at Halls than you are ;)

Halls is (c) 1983 .. 2016 Simon Brattel, Neil Mottershead and Martin Horsley.


To assemble these sources you need Zeus.exe - it is free and can be downloaded
from:

http://www.desdes.com/products/oldfiles/zeus.exe

All you should need to do is put all these files in a single directory, with
Zeus.exe if you want - run zeus.exe and open "front.asm" (this file)

Edit the line below to point the output directory somewhere safe on your
system and then hit the assemble button. That's it...

Oh - if you do anything interesting with these sources, let me know:
crem@desdes.com should reach me. Or search ye webbie for "Simon Brattel"
and if I've died or something similarly inconvenient you're stuffed. And
so, presumably, am I.

Note: Amuses me that the changes I've made to this are in much the same
rotten style as the original. Poor formatting must be contagious...

---------------------------------------------------------------------------------------*/

; Show the memory contents in separate memory panels

;                        zeusmem $C000,"Maze",128,true,false
;                        zeusmem $4000,"Screen",32,true,false
;                        zeusmem $5800,"Attrs",32,true,false

; Make sure the assembler is competent enough

        if ZEUSVER < 21 ; Understands the emulation options
          zeuserror "You need a more recent version of zeus to use these sources, please update it."
          zeuserror "http://www.desdes.com/products/oldfiles/zeus.exe"
        endif

; Load the stimulation file.
;                        Zeus_StimDataErase = true               ; Clear any existing simulation file
;                        Zeus_StimDataLoad = "HallsSim.txt"      ; So we always load this one

                        Zeus_TapeFileLoad = "hott.tzx"  ; Setup for loading this tape file

; Change this to point somewhere suitable on your computer.
; Note that Windows 7 doesn't let you write into the root directory!

; Some testing options

;bFixedMaze              equ true                        ; Same one every time
;bMyDefaults             equ true                        ; For testing
;bReturnInstantResult    equ true                        ; For testing
;bNoDamage               equ true                        ; Don't damage the man

                        include "equ.inc"               ; Equates

                        emulate_spectrum                ; Tell the emulator what we are

; Start of Spectrum free memory

        profile=true

                        org $5B00                       ; Immediately after the screen

TapeSaveRegion1Start    equ *                           ; The first block

                        ; Include the fixed data and the start vector

                        include "chars.inc"           ; The various character sets
;                        include "chars_3.inc"           ; The various character sets
;                        include "chars_wide.inc"           ; The various character sets
;                        include "chars_curly.inc"           ; The various character sets

mAlignRandomFill        macro(Alignment)

                          while ( * mod Alignment) <> 0
                            db 0
                            wend
                          mend

MaxHighScoreText        equ 104                         ; No more than this many characters
MaxHighScoreBytes       equ (MaxHighScoreText + 3)*16   ; Allow for some

; A macro to declare highscore table entries

hse                     macro (str,score)
                          db str,0
                          dw score
                          mend

Front_Vars_Start        equ *

HighScores              hse("Martin Horsley - The Good",8085)                   ; He really was, nobody could touch him at playing halls.
                        hse("Neil Mottershead - The Bad",8080)                  ;
                        hse("Simon Brattel - The Ug... what? Oi!",6809)         ;
                        hse("Graham Stafford - the betrayer!",6502)             ; He asked for this. Search me...
                        hse("Matt Smith - no, not that one...",6502)            ; He wrote a PC retro remake of halls
                        hse("Andy Stewart - (beautiful remake)",6502)           ; He wrote a perfect PC clone. Love it.
                        hse("Rob Crawford - Loader? What loader?",6502)         ; He's, ah, reading this right now. Hello...
;                        hse("John Marshall - The Keys of Krall",6502)           ; To add this or not to add this?
                        hse(" ",4004)                                           ; Need some lines
                        hse(cBlurb,1802)                                        ; Should have thought of this years ago
                        hse(" ",4004)                                           ; Need some lines

                        ; Special case entries here
                        db "You killed ",cnThings," of the things? Gah...",0
                        dw 1654

                        db cScore," point",cAnEss," - call that a score?",0
                        dw 1654

                        hse(" ",1)
                        hse("For the love of all that's lost",1)
                        hse(" ",1)
                        hse("And now for something completely the same...",1)

HighScoreDataEnd        equ HighScores+MaxHighScoreBytes;

TapeSaveRegion1Length   equ *-TapeSaveRegion1Start      ; The first block

                           ; The first block


                        ; Fill the buffer with zeros so it will compress nicely

                        while . < (HighScoreDataEnd - (sBlurbEnd - sBlurbStart))
                          db 0;ZEUSRAND and $FF
                          wend

                        ; We want to use this space for extra messages

nBlurbs                 = 0

blb                     macro (str)
                          dbx *,str,0
                          ::nBlurbs = ::nBlurbs+1
                          mend


sBlurbStart             equ .
TapeSaveRegion2Start    equ *                           ; The second block


BlurbValidMarker        db $1F                          ; A rough check we haven't been overwritten

sBlurbs                 blb("Who did he betray?")
                        blb("Hello? Why am I bad?")
                        blb("Bad at what, exactly?")
                        blb("I'm confused...")
                        blb("Something changed...")
                        blb("Hang on...")

sBlurbEnd               equ .

                        while . < (HighScores+MaxHighScoreBytes)
                          db 0;ZEUSRAND and $FF
                          wend

                        org HighScoreDataEnd

nSomeone                db 0                            ;
nBlurb                  db 0                            ;

; This gets edited

        if def bMyDefaults

OptiData                DEFW 0,sMazes                   ;
                        DEFB 2                          ; Masked
                        DEFB #60                        ;
                        DEFB %0000 0000                 ;
                        DEFB %0000 0001                 ;
                        DEFB %0000 0010                 ;
                        DEFB %0000 0011                 ;
                        DEFB %0000 0000                 ;
                        DEFW AType                      ;
                        defb 0                          ; Don't redraw
                        DEFS 2                          ;

                        DEFW 0,sMonsters                ;
                        DEFB 4                          ; Faster/Dynamic
                        DEFB #78                        ;
                        DEFB %0000 0000                 ; Slow
                        DEFB %0000 0001                 ; Normal
                        DEFB %0000 0011                 ; Normal+More aggessive ones
                        DEFB %0000 0111                 ; Normal+Mormal+Dynamic
                        DEFB %0000 0000                 ;
                        DEFW BType                      ;
                        defb 0                          ; Don't redraw
                        DEFS 2                          ;

                        defw 0,sNoises                  ;
                        defb 2                          ;
                        defb $60                        ;
                        defb $00,$10,$00,$00,$00        ;
                        defw CType                      ;
                        defb 0                          ; Don't redraw
                        DEFS 2                          ;

                        defw 0,sKeys                    ;
                        defb 1                          ;
                        defb $60                        ;
                        defb $00,$01,$02,$03,$04        ;
                        defw DType                      ;
                        defb 1                          ; Redraw
                        DEFS 2                          ;

        else

OptiData                DEFW 0,sMazes                   ; Normal
                        DEFB 1                          ;
                        DEFB #60                        ;
                        DEFB %0000 0000                 ;
                        DEFB %0000 0001                 ;
                        DEFB %0000 0010                 ;
                        DEFB %0000 0011                 ;
                        DEFB %0000 0000                 ;
                        DEFW AType                      ;
                        defb 0                          ; Don't redraw
                        DEFS 2                          ;

                        DEFW 0,sMonsters                ;
                        DEFB 2                          ;
                        DEFB #78                        ;
                        DEFB %0000 0000                 ; Slow
                        DEFB %0000 0001                 ; Normal
                        DEFB %0000 0011                 ; Normal+More aggessive ones
                        DEFB %0000 0111                 ; Normal+Mormal+Dynamic
                        DEFB %0000 0000                 ;
                        DEFW BType                      ;
                        defb 0                          ; Don't redraw
                        DEFS 2                          ;

                        defw 0,sNoises                  ;
                        defb 2                          ;
                        defb $60                        ;
                        defb $00,$10,$00,$00,$00        ;
                        defw CType                      ;
                        defb 0                          ; Don't redraw
                        DEFS 2                          ;

                        defw 0,sKeys                    ;
                        defb 1                          ;
                        defb $60                        ;
                        defb $00,$01,$02,$03,$04        ;
                        defw DType                      ;
                        defb 1                          ; Redraw
                        DEFS 2                          ;

        endif

nOptions                equ ( * - OptiData )/16         ; This many

OptKeyTab               defb $FF,$FF,$84,$00            ; Esc
                        defb $F7,$FE,$00,$01            ; "1"
                        defb $F7,$FD,$00,$02            ; "2"
                        defb $F7,$FB,$00,$03            ; "3"
                        defb $F7,$F7,$00,$04            ; "4"
                        defb $F7,$EF,$00,$05            ; "5"
                        defb $EF,$EF,$84,$06            ; "6"
                        defb $EF,$F7,$84,$07            ; "7"
                        defb $EF,$FB,$84,$08            ; "8"
                        defb $EF,$FD,$84,$09            ; "9"
                        defb $BF,$FE,$84,$0A            ; cCret
                        defb $7F,$FE,$00,$0B            ; " "

sHighC                  defb $07
HSChar                  defb "1",$92,$03,$00

Front_Vars_End          equ *

; Key descriptions

pKeyNames_ZXNM          equ * - ckLeft                  ; Base to be added
                        db "ZXNMFLASDC"                 ; Normal keys
pKeyNames_ADWS          equ * - ckLeft                  ; Base to be added
                        db "ADWSKLJINM"                 ; Alternate keys

; Front end code starts here

Start                   di                              ;

Boot                    ld sp,word StackTop             ; Set the stack

                        call InitKbd                    ; Setup the keys

                        call InitOpti                   ; And the options

DispMain                di                              ; Start the front end

                        ld hl,IntPage                   ; Configure it
                        ld a,$BF                        ; Set it to BF
DMLp                    ld (hl),a                       ;
                        inc l                           ;
                        jr nz DMLp                      ;
                        inc h                           ;
                        ld (hl),a                       ;

                        ld a,IntPage/256                ;
                        ld i,a                          ;
                        im 2                            ;
                        ei                              ; EI

                        ld hl,sMainMenu                 ; Show the menu
                        call PrintStr                   ;

                        ld hl,HSTimeOut                 ; Pause before show highscores

MainLoop                ld bc,$7810                     ;
                        call ScanOptiKeys               ;

                        ld de,MainVecTab                ;
                        jp c,VecGoto                    ;

                        dec hl                          ; Loop
                        ld a,h                          ;
                        or l                            ;
                        jr nz,MainLoop                  ;

                        ; Show the highscore table

DispHS                  ld hl,0                         ;
                        ld (GameScore),hl               ;

DispHS1                 xor a                           ;
                        ld (EntryF),a                   ;

                        ld hl,sHighA                    ;
                        call PrintStr                   ;

                        ld ix,HighScores                ;
                        ld a,"1"                        ;
DHSlp                   call PrintHS                    ;

                        inc a                           ;
                        cp "9"+1                        ;
                        jr nz,DHS1                      ;

                        add a,"A"-("9"+1)               ;

DHS1                    cp "H"                          ;
                        jr nz,DHSlp                     ;

                        ld a,(EntryF)                   ;
                        or a                            ;
                        jp nz,AcceptEnt                 ;

                        ld hl,sHighB                    ;
                        call PrintStr                   ;

HSw                     ld bc,$0030                     ;
                        call ScanOptiKeys               ;

                        jr nc,HSw                       ;

                        cp 10                           ;
                        jp z,DispMain                   ;

                        ; Now, play whatever game type we last played

                        ld a,(GameType)                 ;
                        or a                            ;
                        jp nz DispRunningGame           ;

                        jp DispGame                     ;

; Print a character and expand the special ones

PrintSmart              cp cBlurb                       ; Is it a blurb?
                        jp z PS_Blurb                   ; Yes, skip

                        cp cAnEss                       ; Is it an 'S' (last printed number was not 1)
                        jp z PS_AnEss                   ; Yes, skip

                        cp cnThings                     ; Thing-count
                        jr z PS_nThings                 ; Yes, skip

                        cp cnRings                      ; Ring-count
                        jr z PS_nRings                  ; Yes, skip

                        cp cScore                       ; Score
                        jr z PS_Score                   ; Yes, skip

                        cp cTabMid                      ; Align
                        jr z PS_TabMid                  ;

                        cp cEnableJokes                 ; Enable them?
                        jp z PS_EnableJokes             ; Yes

                        cp cEnablePiss                  ; Enable them?
                        jp z PS_EnablePiss              ; Yes

                        cp ckLeft                       ; A key-name?
                        jp c Print                      ; No, print it

                        push hl                         ; Save
                        push de

                        ld e,a                          ; Save

                        ld hl,pKeyNames_ZXNM            ;
                        ld a,(DType)                    ;
                        or a                            ;
                        jr z PS_Key                     ;
                        ld hl,pKeyNames_ADWS            ;

PS_Key                  ld a,e                          ;
                        pop de                          ;

                        add a,l                         ;
                        ld l,a                          ;

                        adc a,h                         ;
                        sub l                           ;
                        ld h,a                          ;

                        ld a,(hl)                       ;
                        pop hl                          ;

                        jp Print                        ;

PS_TabMid               ld a,128                        ;
                        ld (pCurX),a                    ;
                        ret                             ;

PS_nThings              push hl                         ;

                        ld hl,(nDTs)                    ;
                        ld a,l                          ;
                        or h                            ;
                        jr z PS_None                    ;

                        ld a,h                          ;
                        xor 1                           ;
                        or l                            ; a=0 if hl=256
                        jr nz PS_Decimal                ;

                        ; We want to say "all"

                        call PrintStrFollowing          ;
                        noflow                          ; Tell the assembler this is valid
                        db "all",0                      ;

                        jr PS_StrPlusal                 ; Set the S flag

                ; None

PS_None                 ld a,1                          ;
                        ld (AnEssFlag),a                ; Set the "S" flag

                        call PrintStrFollowing          ;
                        noflow                          ; Tell the assembler this is valid
                        db "none",0                     ;

PS_StrPlusal            ld a,1                          ;
                        ld (AnEssFlag),a                ; Set the "S" flag

                        pop hl
                        ret

PS_nRings               ld a,(nRings)                   ;

PS_A                    push hl                         ;
                        ld l,a                          ;
                        ld h,0                          ;
                        jr PS_Decimal                   ;

PS_Score                push hl                         ;

                        ld hl,(LastScore)               ; Get the score

                        ; Show the count and set the S flag

PS_Decimal              ld a,l                          ;
                        xor 1                           ; a=0 if l=1
                        or h                            ; a=0 if hl=1
                        ld (AnEssFlag),a                ;

                        call PrintDec_NLZ               ; Print it

                        pop hl                          ; Restore
                        ret                             ; Done

                ; Show an S iff required

PS_AnEss                ld a,(AnEssFlag)                ;
                        or a                            ;
                        ret z                           ;

                        ld a,"s"                        ;
                        jr Print                        ;

                ; Generate a new blurb number

ChooseNewBlurb          ld a,nBlurbs                    ; Get a 0..1-N
                        call RandRange                  ;
                        ld (nBlurb),a                   ;
                        ret                             ; Done

                ; Show a blurb instead

PS_Blurb                push hl                         ; Save
                        push bc                         ;

                        ; Are they valid?

                        ld a,(BlurbValidMarker)         ; Check a very simple flag
                        cp $1F                          ;
                        jr nz PHS_NoBlurbs              ; Overwritten, ignore

                        ; Get the requisite blurb text

                        ld a,(nBlurb)                   ; Get it
                        inc a                           ;
                        ld b,a                          ;

                        ; Find the blurb

                        ld hl,sBlurbs                   ;

PHS_SkipLp              djnz PHS_SkipLp2                ;

                        jr PHS_PrintEncStr              ;

sNoBlurb                dbx *,"Hmmm...",0               ;

                ; There aren't any, show a dummy

PHS_NoBlurbs            ld hl,sNoBlurb                  ;
                        jr PHS_PrintEncStr              ;

                ; Print an encyrpted string

PrintEncStr             push hl                         ;
                        push bc                         ;

PHS_PrintEncStr         ld c,l                          ;

                        ; Print this entry

PHS_Lp                  ld a,(hl)                       ;
                        call Decrypt_C                  ;
                        jr z PSB_X                      ; Done

                        inc hl                          ;
                        call Print                      ;
                        jr PHS_Lp                       ;

PSB_X                   pop bc                          ;
                        pop hl                          ;
                        ret                             ;

Decrypt_C               xor c                           ;

                        push af                         ;
                        ld a,c                          ;
                        add a,encoff                    ;
                        ld c,a                          ;
                        pop af                          ;
                        ret                             ;

                ; Skip an entry it

PHS_SkipLp2             ld c,l                          ; Decrypt

PHS_SkipLp2Lp           ld a,(hl)                       ;
                        inc hl                          ;
                        call Decrypt_C                  ;
                        jr z PHS_SkipLp                 ; Entry skipped

                        jr PHS_SkipLp2Lp                ;

; Print a character

Print                   push ix                         ;
                        push hl                         ;
                        push de                         ;
                        push bc                         ;
                        push af                         ;

                        cp cWhite+1                     ;
                        jr c,pColour                    ;

                        cp cCls                         ;
                        jr z,pCls                       ;

                        cp cCret                        ;
                        jr z,pCR                        ;

                        cp Home                         ;
                        jr z,pHome                      ;

                        ld h,wChars/256                 ;
                        ld l,a                          ;

                        ld a,(pCurX)                    ;
                        add a,(hl)                      ;
                        call c,pCret                    ;

                        call pOutChar                   ;

                        ld a,(pCurX)                    ;
                        add a,(hl)                      ;
                        ld (pCurX),a                    ;

pExit                   pop af                          ;
                        pop bc                          ;
                        pop de                          ;
                        pop hl                          ;
                        pop ix                          ;
                        ret                             ;

;

pColour                 cp 4                            ;
                        jr nc,pCol1                     ;

                        or $40                          ;

pCol1                   ld (CurCol),a                   ;
                        jr pExit                        ;

pCR                     call pCret                      ;
                        jr pExit                        ;

;

pCls                    ld e,$00                        ;
                        ld hl,Attrs                     ;
                        ld bc,$0300                     ;

pC1                     ld a,(hl)                       ;
                        and $07                         ;
                        jr z,pC2                        ;

                        set 0,e                         ;
                        dec (hl)                        ;
pC2                     cpi                             ;
                        jp pe,pC1                       ;

                        call FrontDelay                 ;

                        bit 0,e                         ;
                        jr nz,pCls                      ;

                        ld hl,ScreenBase                ;
                        ld de,ScreenBase+1              ;
                        ld bc,$1800                     ;

                        xor a                           ;
                        ld (hl),a                       ;

                        ldir                            ;

                        ld a,(CurCol)                   ;
                        ld (hl),a                       ;

                        ld bc,$02FF                     ;

                        ldir                            ;

                        xor a                           ;
                        out ($FE),a                     ;

pHome                   xor a                           ;
                        ld (pCurX),a                    ;
                        ld (ppCurY),a                   ;

                        ld a,cWhite                     ;
                        ld (CurCol),a                   ;

                        ld hl,ScreenBase                ;
                        ld (pCurY),hl                   ;

                        jr pExit                        ;

pCret                   ld a,(ppCurY)                   ;
                        inc a                           ;

SetUpY                  ld (ppCurY),a                   ;

                        and $1F                         ;

                        push hl                         ;

                        ld l,a                          ;
                        ld h,$00                        ;

                        add hl,hl                       ;

                        ld de,YTableFE                  ;
                        add hl,de                       ;

                        ld a,(hl)                       ;
                        inc hl                          ;
                        ld h,(hl)                       ;
                        ld l,a                          ;

                        ld (pCurY),hl                   ;

                        xor a                           ;
                        ld (pCurX),a                    ;

                        pop hl                          ;
                        ret                             ;

pOutChar                bit 7,l                         ;
                        ret nz                          ;

                        push hl                         ;

                        ld h,0                          ;

                        add hl,hl                       ;
                        add hl,hl                       ;
                        add hl,hl                       ;

                        ld de,pChars-256                ;
                        add hl,de                       ;

                        push hl                         ;
                        pop ix                          ;

                        ld a,(pCurX)                    ;

                        rrca                            ;
                        rrca                            ;
                        rrca                            ;
                        and $1F                         ;

                        ld de,(pCurY)                   ;

                        or e                            ;
                        ld e,a                          ;

                        push de                         ;

                        ld a,d                          ;
                        rrca                            ;
                        rrca                            ;
                        rrca                            ;
                        and $03                         ;
                        or $58                          ;
                        ld d,a                          ;

                        ld a,(CurCol)                   ;
                        ld (de),a                       ;
                        inc de                          ;
                        ld (de),a                       ;

                        pop de                          ;

                        ld a,(pCurX)                    ;
                        and $07                         ;
                        ld c,a                          ;

                        ld b,$08                        ;

OutCharL                ld h,(ix)                       ;
                        ld l,$00                        ;

                        ld a,c                          ;
                        or a                            ;
                        jr z,OCL1                       ;

OCL2                    srl h                           ;
                        rr l                            ;
                        dec a                           ;
                        jr nz,OCL2                      ;

OCL1                    ld a,(de)                       ;
                        xor h                           ;
                        ld (de),a                       ;

                        inc e                           ;

                        ld a,(de)                       ;
                        xor l                           ;
                        ld (de),a                       ;

                        dec e                           ;

                        inc ix                          ;
                        inc d                           ;

                        djnz OutCharL                   ;

                        pop hl                          ;
                        ret                             ;

YTableFE                defw $4000,$4020,$4040,$4060    ;
                        defw $4080,$40A0,$40C0,$40E0    ;
                        defw $4800,$4820,$4840,$4860    ;
                        defw $4880,$48A0,$48C0,$48E0    ;
                        defw $5000,$5020,$5040,$5060    ;
                        defw $5080,$50A0,$50C0,$50E0    ;
                        defw 0,0,0,0,0,0,0,0            ;

  ;      if false
Wait                    ld a,$7F                        ;
                        in a,($FE)                      ;
                        rra                             ;
                        jr nc,Wait                      ;
                        ret                             ;
   ;     endif

FrontDelay              push bc                         ;
                        ld bc,$0320                     ;
Del1                    cpi                             ;
                        dec hl                          ;
                        jp pe,Del1                      ;
                        pop bc                          ;
                        ret                             ;

PrintStr                push hl                         ;
                        push af                         ;
PStr1                   ld a,(hl)                       ;
                        inc hl                          ;
                        or a                            ;
                        jr z,PStr2                      ;
                        call PrintSmart                 ;
                        jr PStr1                        ;

PStr2                   pop af                          ;
                        pop hl                          ;
                        ret                             ;

PrintStrFollowing       ex (sp),hl                      ;
                        push af                         ;
PStr11                  ld a,(hl)                       ;
                        inc hl                          ;
                        or a                            ;
                        jr z,PStr12                     ;
                        call PrintSmart                 ;
                        jr PStr11                       ;

PStr12                  pop af                          ;
                        ex (sp),hl                      ;
                        ret                             ;

InitKbd                 ld hl,KeyCounts                 ;
                        ld de,KeyCounts+1               ;
                        ld bc,$0027                     ;
                        ld (hl),$49                     ;
                        ldir                            ;

                        xor a                           ;
                        ld (CurrKey),a                  ;

                        ld hl,20                        ;
                        ld (RepSpeed1),hl               ;

                        ld hl,3                         ;
                        ld (RepSpeed2),hl               ;
                        ret                             ;

ReadKey                 push ix                         ;
                        push hl                         ;
                        push de                         ;
                        push bc                         ;
                        push af                         ;

                        ex af,af'                       ;
                        push af                         ;

                        ld hl,CapsShift                 ;
                        set 7,(hl)                      ;

                        ld hl,SymbolShift               ;
                        set 7,(hl)                      ;

                        call ReadKeys                   ;

                        dec a                           ;
                        ld b,a                          ;

                        xor a                           ;

                        ld hl,CapsShift                 ;
                        bit 6,(hl)                      ;
                        jr z,Shifta                     ;

                        add a,40                        ;

Shifta                  ld hl,SymbolShift               ;
                        bit 6,(hl)                      ;
                        jr z,Shiftb                     ;

                        add a,80                        ;

Shiftb                  add a,b                         ;

                        ld e,a                          ;
                        ld d,0                          ;

                        ld hl,kTab0                     ;
                        add hl,de                       ;

                        pop af                          ;
                        ex af,af'                       ;

                        pop af                          ;

                        ld a,(hl)                       ;

                        pop bc                          ;
                        pop de                          ;
                        pop hl                          ;
                        pop ix                          ;
                        ret                             ;

ReadKeys                ld a,1                          ;
                        ld (CursorSpeed),a              ;
                        ld (CursorType),a               ;

ReadKeyLp               call CursorInv                  ;

                        call ScanKeys                   ;

                        jr nz,RKNewKey                  ;

                        ld a,(CurrKey)                  ;
                        or a                            ;
                        jr z,ReadKeyLp                  ;

                        ld hl,(CurrKeyAddr)             ;
                        bit 6,(hl)                      ;
                        jr z,RKCancelOld                ;

                        ld hl,(RepDel)                  ;
                        dec hl                          ;
                        ld (RepDel),hl                  ;

                        ld a,l                          ;
                        or h                            ;
                        jr nz,ReadKeyLp                 ;

                        ld hl,(RepSpeed2)               ;
RK1                     ld (RepDel),hl                  ;

                        call CursorOff                  ;

                        ld a,(CurrKey)                  ;
                        ret                             ;

RKNewKey                ld (CurrKey),a                  ;

                        ld hl,(RepSpeed1)               ;
                        ld (CurrKeyAddr),ix             ;

                        jr RK1                          ;

RKCancelOld             xor a                           ;
                        ld (CurrKey),a                  ;

                        jr ReadKeyLp                    ;

CursorOff               ld a,(CursorType)               ;
                        or a                            ;
                        jr z,FlipCur                    ;
                        ret                             ;

CursorInv               ld a,(CursorSpeed)              ;
                        dec a                           ;
                        ld (CursorSpeed),a              ;
                        ret nz                          ;

                        ld a,FlashRate                  ;
                        ld (CursorSpeed),a              ;

                        ld a,(CursorType)               ;
                        xor $01                         ;
                        ld (CursorType),a               ;

FlipCur                 ld l,$5F                        ;

                        jp pOutChar                     ;

ScanKeys                ld bc,$FFFE                     ;
                        ld hl,KeyCounts                 ;
                        ld e,0                          ;

SK_Frame                ld a,(IrqFlag)                  ;
                        or a                            ;
                        jr z SK_Frame                   ;

                        xor a                           ;
                        ld (IrqFlag),a                  ;

ScanKeyLp               dec b                           ;
                        in a,(c)                        ;

                        ld d,5                          ;

ScanKeyLp2              rra                             ;
                        call nc,SKPressed               ;
                        call c,SKNotPressed             ;
                        inc hl                          ;

                        dec d                           ;
                        jr nz,ScanKeyLp2                ;

                        scf                             ;
                        rl b                            ;
                        inc b                           ;
                        jr nz,ScanKeyLp                 ;

                        ld a,e                          ;
                        or a                            ;
                        ret                             ;

SKPressed               ex af,af'                       ;

                        ld a,(hl)                       ;
                        and $3F                         ;

                        ; Has it been pressed enough? Is it a shift key?

                        push bc                         ;
                        push af                         ;

                        ld c,MinScansShift              ; Assume shift key

                        ld a,l                          ;
                        cp low SymbolShift              ;
                        jr z SK_GotScans                ;
                        cp low CapsShift                ;
                        jr z SK_GotScans                ;

                        ld c,MinScans                   ;

SK_GotScans             pop af                          ;
                        cp c                            ; MinScans or MinScansShift
                        pop bc                          ; Restore
                        jr z,SKP1                       ;

                        inc (hl)                        ;

                        ex af,af'                       ;
                        ret                             ;

SKP1                    bit 6,(hl)                      ;
                        jr nz,SKexit                    ;

                        bit 7,(hl)                      ;
                        jr nz,SKexit1                   ;

                        ld a,e                          ;
                        or a                            ;
                        jr nz,SKexit                    ;

                        push hl                         ;
                        pop ix                          ;

                        call CalcKeyPos                 ;

                        ld e,a                          ;

SKexit1                 set 6,(hl)                      ;

SKexit                  ex af,af'                       ;
                        ret                             ;

SKNotPressed            ex af,af'                       ;

                        ld a,(hl)                       ;
                        and $3F                         ;
                        jr z,SKNP1                      ;

                        dec (hl)                        ;

                        ex af,af'                       ;
                        ret                             ;

SKNP1                   res 6,(hl)                      ;

                        ex af,af'                       ;
                        ret                             ;

CalcKeyPos              push de                         ;

                        ld de,KeyCounts-1               ;
                        or a                            ;
                        sbc hl,de                       ;

                        ld a,l                          ;

                        add hl,de                       ;

                        pop de                          ;
                        ret                             ;

; Keyboard map

kTab0                   defb " zxcv"                    ;
                        defb "asdfg"                    ;
                        defb "qwert"                    ;
                        defb "12345"                    ;
                        defb "09876"                    ;
                        defb "poiuy"                    ;
                        defb cCret,"lkjh"                ;
                        defb "  mnb"                    ;

                        defb " ZXCV"                    ;
                        defb "ASDFG"                    ;
                        defb "QWERT"                    ;
                        defb "!@#$%"                    ;
                        defb cDel,")('&"                ;
                        defb "POIUY"                    ;
                        defb cCret,"LKJH"               ;
                        defb "  MNB"                    ;

                        defb " :#?/"                    ;
                        defb "~|'{}"                    ;
                        defb "   <>"                    ;
                        defb "!@#$%"                    ;
                        defb "_)('&"                    ;
                        defb $22,$2B," ]["              ;
                        defb cCret,"=+-^"               ;
                        defb "  .,*"                    ;

                        ; Note that the hidden message in the keys was corrupted

                        defb "     "                    ;
                        defb "t I g"                    ;
                        defb "a gir"                    ;
                        defb cBlue,cRed,cMagenta,cGreen,cCyan ;
                        defb Clr,cCls,cCret,cWhite,cYellow;
                        defb "sel l"                    ;
                        defb "dam o"                    ;
                        defb "     "                    ;

SetUpKeyMap             ld hl,KeyCounts                 ;
                        ld b,40                         ;
SUKM                    set 6,(hl)                      ;
                        inc hl                          ;
                        djnz SUKM                       ;
                        ret                             ;

InitOpti                ld ix,OptiData                  ;
                        ld hl,AType                     ;
                        ld b,nOptions                   ;
                        ld d,0                          ;

InitOptiLp              push ix                         ;

                        ld a,(ix+4)                     ; Current option
                        add a,5                         ; + Offset to values
                        ld e,a                          ;

                        add ix,de                       ; Move to it

                        ld a,(ix)                       ; Set the option value
                        ld (hl),a                       ;

                        inc hl                          ; Next one

                        pop ix                          ;

                        ld e,16                         ; Skip to next option menu
                        add ix,de                       ;

                        djnz InitOptiLp                 ;

                        ret                             ;

GetString               ld ix,HSBuffer                  ;
                        call InitKbd                    ;

                        ld a,cMagenta                   ;
                        call Print                      ;

AElp                    call ReadKey                    ;

                        or a                            ;
                        jr z,AElp                       ;

                        cp cCret                        ;
                        ret z                           ;

                        cp cDel                         ;
                        jr z,AEDel                      ;

                        cp Clr                          ;
                        jr z,AEclr                      ;

                        cp 8                            ;
                        jr c,AE1                        ;

                        cp $18                          ;
                        jr c,AElp                       ;

AE1                     ld e,a                          ;
                        ld d,wChars/256                 ;
                        ld a,(de)                       ;
                        ld c,a                          ;

                        ld a,(pCurX)                    ;
                        add a,c                         ;
                        cp 255-39                       ;
                        jr nc,AElp                      ;

                        push ix                         ;
                        pop hl                          ;

                        ld bc,HSBuffer                  ;
                        or a                            ;
                        sbc hl,bc                       ;

                        ld bc,130                       ;
                        or a                            ;
                        sbc hl,bc                       ;
                        jr nc,AElp                      ;

                        ld a,e                          ;
                        ld (ix),a                       ;
                        inc ix                          ;
                        call Print                      ;
                        jr AElp                         ;

AEDel                   call AEcDel                     ;
                        jp AElp                         ;

AEclr                   call AEcDel                     ;
                        jr nz,AEclr                     ;

                        jp AElp                         ;

; Remove a character/colour code

AEcDel                  ld de,HSBuffer                  ;

                        push ix                         ;
                        pop hl                          ;

                        or a                            ;
                        sbc hl,de                       ;
                        add hl,de                       ;

                        ret z                           ;

                        dec hl                          ;
                        ld a,(hl)                       ;
                        cp 8                            ;
                        jr c,DelCC                      ;

                        push hl                         ;
                        ld b,a                          ;

                        ld l,a                          ;
                        ld h,wChars/256                 ;

                        ld e,(hl)                       ;
                        ld d,0                          ;

                        ld hl,(pCurX)                   ;
                        or a                            ;
                        sbc hl,de                       ;

                        ld (pCurX),hl                   ;

                        ld a,b                          ;
                        call Print                      ;

                        ld (pCurX),hl                   ;

                        pop ix                          ;
                        or a                            ;
                        ret                             ;

DelCC                   ld de,HSBuffer                  ;
                        ld c,cMagenta                   ;

DCC1                    or a                            ;
                        sbc hl,de                       ;
                        add hl,de                       ;
                        jr z,DCC2                       ;

                        ld a,(de)                       ;
                        cp 8                            ;
                        jr nc,DCC3                      ;
                        ld c,a                          ;

DCC3                    inc de                          ;
                        jr DCC1                         ;

DCC2                    ld a,c                          ;
                        call Print                      ;

                        push hl                         ;
                        pop ix                          ;

                        or a                            ;
                        ret                             ;

AcceptEnt               ld hl,sAccept                   ;
                        call PrintStr                   ;

                        ld hl,(EntryY)                  ;
                        ld (pCurY),hl                   ;

                        ld a,9                          ;
                        ld (pCurX),a                    ;

                        call GetString                  ;

                        push ix                         ;
                        pop hl                          ;

                        call MWEntry                    ; Modify it

                        ld (hl),0                       ; Terminate it
                        inc hl                          ;
                        ld a,(GameScore)                ;
                        ld (hl),a                       ;
                        inc hl                          ;
                        ld a,(GameScore+1)              ;
                        ld (hl),a                       ;
                        inc hl                          ;
                        ld de,HSBuffer                  ;
                        or a                            ;
                        sbc hl,de                       ;
                        ld b,h                          ;
                        ld c,l                          ;
                        push bc                         ;
                        ld hl,(pHSEnd)                  ;
                        ld de,(pInsert)                 ;
                        or a                            ;
                        sbc hl,de                       ;
                        push hl                         ;
                        ld hl,(pHSEnd)                  ;
                        add hl,bc                       ;
                        ex de,hl                        ;
                        ld hl,(pHSEnd)                  ;
                        pop bc                          ;
                        inc bc                          ;
                        lddr                            ;
                        ld hl,HSBuffer                  ;
                        ld de,(pInsert)                 ;
                        pop bc                          ;
                        ldir                            ;
                        jp DispHS                       ;

; Mess with the hi-score entry text

MWEntry                 ld de,HSBuffer                  ; Is the entry blank?
                        or a                            ;
                        sbc hl,de                       ;
                        jr nz,MWE1                      ;

                        ; If so we change it to "Someone" for the first 8 tries
                        ; then to "Someone else" after 8 blank entries
                        ; then to "Someone else who is shy or lazy" after 16

                        ld a,(nSomeone)                 ;
                        inc a                           ;
                        ld (nSomeone),a                 ;

                        cp 16                           ; > 16?
                        ld bc,SomeoneEnd-Someone        ;
                        jr nc,MWEa                      ;

                        cp Someone1 - Someone           ; < 8
                        ld bc,8                         ;
                        jr c,MWEa                       ;

                        ld bc,Someone2-Someone          ; 9..15

MWEa                    ld hl,Someone                   ;
                        ld de,HSBuffer                  ;
                        ldir                            ;
                        ex de,hl                        ;
                        ret                             ;

Someone                 db "Someone"                    ;
Someone1                db " else"                      ;
Someone2                db " who is shy or lazy"        ;
SomeoneEnd              equ $                           ;

                ; We have some text

MWE1                    xor a                           ; Clear the someone count
                        ld (nSomeone),a                 ;

                        ld a,l                          ; Entry >= 103 chars?
                        add hl,de                       ;
                        cp 103                          ;
                        jr c,MWE2                       ;

                        ; Replace with the long entry text

                        ld hl,TooLong                   ;
                        ld de,HSBuffer                  ;
                        ld bc,lTooLong                  ;
                        ldir                            ;
                        ex de,hl                        ; pEnd
                        ret                             ;

TooLong                 db "What strange behaviour !"   ;
lTooLong                equ $-TooLong                   ;

MWE2                    ld (hl),0                       ; Terminate the entry
                        push hl                         ; Save HL so we can restore it

                        ld hl,vMazeMasks+4              ;
                        call LoadVectorHL               ;
                        push hl
                        pop ix                          ;

                        ld hl,vMazeMasks+6              ;
                        call LoadVectorHL               ;
                        push hl
                        pop iy                          ;

MWElp                   push ix                         ;

                        ld c,ixl                        ; Encrypt value

                        ld hl,HSBuffer                  ;

Mlp1                    ld a,(hl)                       ; Entry has ended?
                        or a                            ;
                        jr z,Mlpf                       ; Yes, skip

                        cp $21                          ;
                        inc hl                          ;
                        jr c,Mlp1                       ;

                        call ConvLC                     ;

                        ld b,a                          ; Save
                        call GetNextComp_A              ; Get the next byte
                        cp b                            ; Same?
                        jr z,Mlp1                       ; Yes, loop

                        ; They're different. We don't consider this a match.

                        jp MlSkipComp                   ; Skip it (NZ)

        ; The entry has ended, has the comparison?

Mlpf                    call GetNextComp_A              ; Get the next comparison byte
                        or a                            ;

MlSkipComp              pop ix                          ;
                        jr z,MWEcg                      ; Replace the text

                        ; Skip to end of comparison text

                        ld c,ixl                        ; Set the encryption
StepIX                  call GetNextComp_A              ;
                        or a                            ;
                        jr nz,StepIX                    ;

                        ; We have skipped the comparison, so skip the replacement

                        ld c,iyl                        ; Setup the encrypt

StepIY                  call GetNextRepl_A              ;
                        or a                            ;
                        jr nz,StepIY                    ;

                        ; Is this the last comparison text?

                        ld c,ixl                        ; Encrypt
                        ld a,(ix)                       ;
                        xor c                           ;

                        inc a                           ; Is this FF?
                        jr nz,MWElp                     ;

                        pop hl                          ; Restore HL (End of text)
                        ret                             ;

ConvLC                  cp "Z"+1                        ;
                        ret nc                          ;
                        cp "A"                          ;
                        ret c                           ;
                        add a,"a"-"A"                   ;
                        ret                             ;

        ; Replace the text in the highscore buffer

MWEcg                   pop af                          ;

                        ld c,iyl                        ; Setup encrypt

                        ld hl,HSBuffer                  ;

CGlp                    call GetNextRepl_A              ; Get the replacement text

                        or a                            ;
                        ret z                           ;

                        ld (hl),a                       ;
                        inc hl                          ;

                        jr CGlp                         ;

; Get comparison text from (IX) into A

GetNextComp_A           ld a,(ix)                       ; Get it
                        inc ix                          ;

                        xor c                           ; Decrypt

                        push af                         ; Change the encrypt
                        ld a,c                          ;
                        add a,encoff                    ;
                        ld c,a                          ;
                        pop af                          ;

                        ret                             ; Done

; Get replacement text from (IY) into A

GetNextRepl_A           ld a,(iy)                       ; Get it
                        inc iy                          ;

                        xor c                           ; Decrypt

                        push af                         ; Change the encrypt
                        ld a,c                          ;
                        add a,encoff                    ;
                        ld c,a                          ;
                        pop af                          ;

                        ret                             ; Done

DispOpti                ld hl,sOptiMenu                 ;
                        call PrintStr                   ;

OptiLoop                ld bc,$78A0                     ; ESC,1..9,cret
                        call ScanOptiKeys               ;
                        jr nc,OptiLoop                  ;

;                cp 5                            ;
;                jp z,HexMon                     ;

                        cp 8                            ;
                        jp z,DispMain                   ;
                        cp 10                           ;
                        jp z,DispMain                   ;

                        dec a                           ;
                        add a,a                         ;
                        add a,a                         ;
                        add a,a                         ;
                        add a,a                         ;
                        ld e,a                          ;
                        ld d,0                          ;
                        ld ix,OptiData                  ;
                        add ix,de                       ;

                        ; We're pointing at the option definition

OptiPrintMenu           ld hl,(ix+2)                    ;
                        call PrintStr                   ;

                        ld a,20                         ;
                        ld (ppCurY),a                   ;

                        ld hl,sBottom                   ;
                        call PrintStr                   ;

OptiLpA                 ld a,(ix+4)                     ;
                        call CalcpVDU                   ;
                        ld (FlashpDisp),hl              ;

                        call GrabLine                   ;

                        ld d,0                          ;

OptiLpB                 ld hl,OptiFlashR                ;

OptiLp1                 ld b,(ix+5)                     ;
                        ld c,$E0                        ;
                        call ScanOptiKeys               ;
                        jr c,OptiKP                     ;

                        dec hl                          ;
                        ld a,l                          ;
                        or h                            ;
                        jr nz,OptiLp1                   ;

                        ld a,d                          ;
                        cpl                             ;
                        ld d,a                          ;
                        or a                            ;

                        push de                         ;
                        ld hl,(FlashpDisp)              ;
                        push af                         ;
                        call z,PutLine                  ;
                        pop af                          ;
                        call nz,ClearLine               ;
                        pop de                          ;

                        jr OptiLpB                      ;

OptiKP                  cp (ix+4)                       ;
                        jr z,OptiLp1                    ;

                        push af                         ;
                        ld hl,(FlashpDisp)              ;
                        call PutLine                    ;
                        pop af                          ;

                        cp 8                            ;
                        jp z,DispMain                   ;
                        cp 9                            ;
                        jp nc,DispOpti                  ;

                        ld (ix+4),a                     ;

                        push ix                         ;
                        pop hl                          ;

                        add a,5                         ;

                        ld e,a                          ;
                        ld d,0                          ;

                        add hl,de                       ;

                        ld a,(hl)                       ;

                        ld hl,(ix+11)                   ;
                        ld (hl),a                       ;

                        ; Redraw the entire menu?

                        ld a,(ix+13)                    ;
                        or a                            ;
                        jr z OptiLpA                    ;

                        jr OptiPrintMenu                ;

CalcpVDU                push de                         ;

                        dec a                           ;

                        ld l,a                          ;
                        ld h,0                          ;

                        add hl,hl                       ;

                        ld de,CpV                       ;
                        add hl,de                       ;

                        ld a,(hl)                       ;
                        inc hl                          ;
                        ld h,(hl)                       ;
                        ld l,a                          ;

                        pop de                          ;
                        ret                             ;

CpV                     defw $5882,$58C2,$5942,$5982,$5A22

GrabLine                ld de,LineBuffer                ;
                        ld bc,30                        ;
                        ldir                            ;
                        ret                             ;

FlashLine               dec hl                          ;

                        ld a,h                          ;
                        or l                            ;
                        ret nz                          ;

                        ld hl,(pKeyVDU)                 ;

                        ld a,(CurType)                  ;
                        xor $01                         ;
                        ld (CurType),a                  ;

                        ld a,(UDKat0)                   ;

                        jr z,FLO                        ;

                        ld a,(UDKat1)                   ;

FLO                     ld hl,(pKeyVDU)                 ;

                        ld de,hl                        ;
                        inc de                          ;

                        ld bc,7                         ;

                        ld (hl),a                       ;

                        ldir                            ;

                        ld hl,UDKfr                     ;
                        ret                             ;

PutLine                 ex de,hl                        ;
                        ld hl,LineBuffer                ;
                        ld bc,30                        ;
                        ldir                            ;
                        ret                             ;

ClearLine               ld e,l                          ;
                        ld d,h                          ;
                        inc de                          ;
                        ld (hl),$41                     ;
                        ld bc,29                        ;
                        ldir                            ;
                        ret                             ;

ClearGraph              call CGa                        ;

                        inc h                           ;
                        ld a,h                          ;
                        and $07                         ;
                        ret z                           ;

                        jr ClearGraph                   ;

CGa                     push hl                         ;

                        ld d,h                          ;
                        ld e,l                          ;
                        inc de                          ;
                        ld bc,$001F                     ;

                        ld (hl),$00                     ;

                        ldir                            ;

                        pop hl                          ;
                        ret                             ;

DispInst                ld hl,sInst                     ;
                        call PrintStr                   ;

DIlp                    ld bc,$7FF0                     ;
                        call ScanOptiKeys               ;
                        jr nc,DIlp                      ;

                        jp DispMain                     ;

PrintHS                 push af                         ;
                        ld (HSChar),a                   ;

                        ld hl,sHighC                    ;
                        call PrintStr                   ;

                        push ix                         ;
                        pop hl                          ;

                        ld a,(EntryF)                   ;
                        or a                            ;
                        jr nz,PHSb                      ;

PHSa                    ld a,(hl)                       ;
                        or a                            ;
                        inc hl                          ;
                        jr nz,PHSa                      ;

                        ld a,(hl)                       ;
                        inc hl                          ;
                        ld h,(hl)                       ;
                        ld l,a                          ;

                        ld de,(GameScore)               ;
                        or a                            ;
                        sbc hl,de                       ;
                        jr nc,PHSb                      ;

                        ld a,1                          ;
                        ld (EntryF),a                   ;

                        ld hl,(pCurY)                   ;
                        ld (EntryY),hl                  ;

                        ld (pInsert),ix                 ;

                        ex de,hl                        ;
                        jp PHSx                         ;

PHSb                    ld a,(ix)                       ;
                        or a                            ;
                        inc ix                          ;
                        jp z,PHS2a                      ;

                        call PrintSmart                 ;
                        jr PHSb                         ;

                ; Show the score

PHS2a                   ld l,(ix)                       ;
                        inc ix                          ;
                        ld h,(ix)                       ;
                        inc ix                          ;
                        ld (pHSEnd),ix                  ;

PHSx                    ld a,255 - (5*6)                ;
                        ld (pCurX),a                    ;

                        ld a,cYellow                    ;
                        call Print                      ;

                        call PrintScoreDec              ;

                        ld a,cCret                      ;
                        call Print                      ;

                        pop af                          ;
                        ret                             ;

PHS2                    inc ix                          ;

PHS3                    call PrintSp                    ;
                        djnz PHS3                       ;

                        jr PHS2a                        ;

PrintSp                 push af                         ;
                        ld a," "                        ;
                        call Print                      ;
                        pop af                          ;
                        ret                             ;

PrintSp2                push af                         ;
                        ld a,$80                        ;
                        call Print                      ;
                        pop af                          ;
                        ret                             ;

PrintDec_NLZ            ld c,1
                        ld de,-10000
                        call PDNL1
                        ld de,-1000
                        call PDNL1
                        ld de,-100
                        call PDNL1
                        ld de,-10
                        call PDNL1
                        LD A,L
                        ADD A,"0"
                        jp Print

PDNL1                   XOR A
PDNL2                   INC A
                        ADD HL,DE
                        JR C,PDNL2
                        SBC HL,DE
                        dec a
                        or "0"

                        CP C
                        JP Z,Print

                        cp "0"
                        ret z

                        ld c,"0"
                        jp Print

PrintScoreDec           ld c,1
                        ld de,-10000
                        call PDD1
                        ld de,-1000
                        call PDD1
                        ld de,-100
                        call PDD1
                        ld de,-10
                        call PDD1
                        LD A,L
                        ADD A,"0"
                        jp Print

PDD1                    XOR A
PDD2                    INC A
                        ADD HL,DE
                        JR C,PDD2
                        SBC HL,DE
                        CP C
                        JR Z,PDDS
                        DEC C
                        ADD A,#2F
                        jp Print

PDDS                    ld a,(pCurX)                    ;
                        add a,6                         ;
                        ld (pCurX),a                    ;
                        ret                             ;

VecGoto                 dec a                           ;
                        add a,a                         ;
                        ld l,a                          ;
                        ld h,0                          ;
                        add hl,de                       ;
                        ld a,(hl)                       ;
                        inc hl                          ;
                        ld h,(hl)                       ;
                        ld l,a                          ;
FrontCallHL             jp (hl)                         ;

MainVecTab              defw DispGame                   ; 1
                        defw DispRunningGame            ; 2
                        defw DispInst                   ; 3
                        defw DispOpti                   ; 4
                        defw DispMain                   ; 5
                        defw DispMain                   ; 6
                        defw DispMain                   ; 7
                        defw DispMain                   ; 8
                        defw DispMain                   ; 9
                        defw DispMain                   ; 10
                        defw DispHS                     ; 11

ScanOptiKeys            push ix                         ;
                        push de                         ;
                        push bc                         ;

                        ; We want the randomize the mazes

                        xor a                           ;
                        ld (IrqFlag),a                  ;

SOK_Frame               push hl                         ;
                        push bc                         ;
                        call Rand                       ;
                        pop bc                          ;
                        pop hl                          ;

                        ld a,(IrqFlag)                  ;
                        or a                            ;
                        jr z SOK_Frame                  ;

                        call SOK                        ;

                        pop bc                          ;
                        pop de                          ;
                        pop ix                          ;
                        ret                             ;

SOK                     ld ix,OptKeyTab                 ;
                        ld e,12                         ;

SOKlp                   rl c                            ;
                        rl b                            ;
                        jr c,SOK0                       ;

                        push bc                         ;
                        ld bc,4                         ;
                        add ix,bc                       ;
                        pop bc                          ;

SOKx                    dec e                           ;
                        jr nz,SOKlp                     ;

                        or a                            ;
                        ret                             ;

SOK0                    call SOK1                       ;
                        ret c                           ;

                        jr SOKx                         ;

SOK1                    push bc                         ;

                        ld b,(ix)                       ;
                        ld c,$FE                        ;
                        in a,(c)                        ;

                        or (ix+1)                       ;
                        inc a                           ;

                        jr z,SOKnp                      ;

                        ld a,(ix+2)                     ;
                        and $0F                         ;
                        cp 1                            ;
                        jr z,SOK2                       ;

                        inc (ix+2)                      ;

SOKxf                   ld bc,4                         ;
                        add ix,bc                       ;
                        or a                            ;

                        pop bc                          ;
                        ret                             ;

SOK2                    bit 7,(ix+2)                    ;
                        jr nz,SOKxf                     ;

                        set 7,(ix+2)                    ;

                        ld a,(ix+3)                     ;

                        ld bc,4                         ;
                        add ix,bc                       ;

                        scf                             ;
                        pop bc                          ;
                        ret                             ;

SOKnp                   ld a,(ix+2)                     ;
                        and $0F                         ;
                        jr z,SOKxf                      ;

                        dec (ix+2)                      ;

                        dec a                           ;
                        and $0F                         ;
                        jr nz,SOKxf                     ;

                        res 7,(ix+2)                    ;

                        jr SOKxf                        ;

DispGame                ld a,cCls                       ;
                        call Print                      ;

                        ; Call the game and get the score back

                        call StartGame                  ; Play a game

                        ; Now, if A=1 we won...

GameOver                call ShowResults                ; Show a report

                        call InitKbd                    ; Setup the keys

                        call ChooseNewBlurb             ; Randomise them

                        JP DispHS1

; The running game

DispRunningGame         ld a,cCls                       ;
                        call Print                      ;

                        ; Call the game and get the score back

                        call StartRunningGame           ; Play a running game

                        ld hl,(GameScore)               ; Halve the score
                        srl h                           ;
                        rrc l                           ;
                        ld (GameScore),hl               ;

                        jr GameOver                     ; As above

; Show the results of a game dead/nRings/score

; If won, show appropriate winning text depending on monsters
; If lost show appropriate insult

ShowResults             ld hl,(GameScore)               ; The score
                        ld (LastScore),hl               ; Remember it

                        or a                            ; Won flag?
                        jr nz SR_Won                    ; Yes, skip

                        ; We lost

                        ld hl,sEndLost                  ;
                        call PrintStr                   ;

                        jr SR_Wait                      ; Wait for enter

                ; We won

SR_Won                  ld hl,sEndWon                   ;
                        call PrintStr                   ;

                        ; Add a comment if the options weren't hard enough

                        ld a,(GameType)                 ; Playing a running game?
                        or a                            ;
                        jr nz SR_NoComment              ;

                        ; Normal game

                        ld a,(BType)                    ; Are we on the easy settings?
                        ld b,a                          ;
                        ld a,(AType)                    ;
                        or b                            ;
                        call z SRW_TryHarder            ; Easy

                        ld a,(BType)                    ; Are we on the really hard monsters?
                        cp %111                         ;
                        call z SRW_WellDone             ; Full-on

                        ; Enter

SR_NoComment            ld hl,sPressEnter               ;
                        call PrintStr                   ;

                        ; Fall into

SR_Wait                 ld bc,$0030                     ;
                        call ScanOptiKeys               ;

                        cp 10                           ;
                        ret z                           ;

                        jr SR_Wait                      ; Loop

SRW_TryHarder           ld hl,sTryHarder                ;
                        jp PrintStr                     ;

SRW_WellDone            ld hl,vMazeMasks+2              ;
                        call LoadVectorHL               ;
                        jp PrintEncStr                  ;

; Given HL as a pVector, find the object

LoadVectorHL            push de                         ;

                        ld e,(hl)                       ;
                        inc hl                          ;
                        ld d,(hl)                       ;
                        dec hl                          ;

                        add hl,de                       ;

                        pop de
                        ret                             ;

; Menu text

sEndLost                defb cCls,cCret,cCret,cYellow
                        defb "Hah! You were defeated...",cCret,cCret
                        db cCyan
                        db "You killed ",cnThings," of the things",cCret
                        db "You collected ",cnRings," ring",cAnEss,cCret
                        db "And you somehow scored ",cScore," point",cAnEss,cCret,cCret

                        ; Fall into...

sPressEnter             db cMagenta,"(Press Enter)"
                        defb cWhite,0

sTryHarder              defb "Now try it with harder monsters",cCret
                        defb "and/or masked mazes if you dare...",cCret,cCret
                        defb "Use the option menu to set them",cCret,cCret
                        defb 0

sEndWon                 defb cCls,cCret,cCret,cYellow
                        defb "Oi! Neil? Martin? They've beaten our Things...",cCret
                        defb "I told you they weren't nasty enough yet!",cCret,cCret
                        db cCyan
                        db "You killed ",cnThings," of the poor things",cCret
                        db "You collected all the rings and the key",cCret
                        db "And you scored ",cScore," point",cAnEss,cCret,cCret
                        db "I suppose we should say congratulations...",cCret,cCret
                        db 0

sMainMenu               defb $07,$0C,$86,$86
                        defb "Halls Of The Things"
                        defb $0D,$0D
;                        defb cCyan,$86,$86,$83,"2011 release"
;                        defb cCyan,$86,$86,$83,"FINAL TEST"
                        defb $0D,$0D,$0D
                        defb $07
                        defb " 1 "
                        defb cCyan
                        defb " Play the standard game"
                        defb $07
                        defb $0D,$0D,$0D
                        defb " 2 "
                        defb $03
                        defb " Play blitz-halls "
                        defb $07
//                        defb cRed,"wild uncontrolled bursts!"
                        defb $0D,$0D
                        defb $0D
                        defb " 3 "
                        defb $06
                        defb " Instructions"
                        defb $0D,$0D
                        defb $0D
                        defb $07
                        defb " 4 "
                        defb $02
                        defb " Change the game options"
                        db $0D
                        defb $0D,$0D
                        defb $0D,$0D
                        defb $0D,$0D
                        defb $03
                        defb "Designed without any aid from the computer."
                        defb $0D,$0D
                        defb "(c) Design Design Software"
                        defb $00

sOptiMenu               defb $0C,$86,$86,$84,$04,"Game Options",$0D,$0D,$0D,$0D,$07
                        defb " 1  ",$05,"Change The Mazes",$0D,$0D,$07
                        defb " 2  ",$06,"Change The Game Speed",$0D,$0D,$0D,$0D,$07
                        defb " 3  ",$03,"Change The Noises",$0D,$0D,$07
                        defb " 4  ",$02,"Change The Key Layout"

                        loop 8
                          db cCret
                          lend

                        ; Fall into...

sBottom                 defb $0D,$07," 8  ",$05,"Return to main menu",$0D,$0D,$07
                        defb " 9  ",$03,"Return to options menu",$00

sMazes                  DEFB $0C
                        DEFB #86,#86,#86
                        DEFM "Maze Options"
                        DEFB cCret,cCret
                        DEFB cCret,cCret,$07
                        DEFM " 1  ",$05
                        DEFM "Normal",$07
                        DEFB cCret,cCret
                        DEFM " 2  ",$06
                        DEFM "Masked",$07
                        DEFB 0

sMonsters               DEFB $0C
                        DEFB #86,#86,#86
                        DEFM "Game Speed"
                        DEFB cCret,cCret
                        DEFB cCret,cCret,$07
                        DEFM " 1  ",$05
                        DEFM "Slower",$07
                        DEFB cCret,cCret
                        DEFM " 2  ",$06
                        DEFM "Normal",$07
                        DEFB cCret,cCret,cCret,cCret
                        DEFM " 3  ",$07
                        DEFM "More aggressive",$07
                        DEFB cCret,cCret
                        DEFM " 4  ",$04
                        DEFM "Really, really annoyed",$07
                        DEFB 0


sNoises                 defb $0C,$86,$86,$86,$82,$06,"Noises",$0D,$0D,$0D,$0D,$07
                        defb " 1  ",$05,"None",$0D,$0D,$07
                        defb " 2  ",$06,"Awful",$00

cKEY                    equ cWhite
cTXT                    equ cYellow

sKeys                   defb $0C,$86,$86,$84,"Key Layout",$0D,$0D,$0D,$0D,$07
                        defb " 1  ",$05,"Normal ZXNM, etc",$0D,$0D,$07
                        defb " 2  ",$06,"Rob's WASD, etc",cCret,cCret
                        defb cCyan," Movement keys",cWhite,cCret
                        defb cKEY," ",ckLeft," ",cTXT,"- Move LEFT",cTabMid,cKEY,ckRight," ",cTXT,"- Move RIGHT",cCret
                        defb cKEY," ",ckUp," ",cTXT,"- Move UP",cTabMid,cKEY,ckDown," ",cTXT,"- Move DOWN",cCret,cCret
                        defb cCyan," Weapon keys",cWhite,cCret
                        defb cKEY," ",ckFireball," ",cTXT,"- Fire fireball",cTabMid,cKEY,ckLightning," ",cTXT,"- Fire lightning",cCret
                        defb cKEY," ",ckArrow," ",cTXT,"- Fire arrow",cTabMid,cKEY,ckSword," ",cTXT,"- Use sword",cCret,cCret
                        defb cCyan," Misc. keys",cWhite,cCret
                        defb cKEY," Space ",cTXT,"- Center man",cTabMid,cKEY,"1 ",cTXT,"- show status",cCret
                        defb cKEY," ",ckDrop," ",cTXT,"- Drop item",cTabMid,cKEY,ckClose," ",cTXT,"- Close a door",cCret,cCret
                        defb 0

sInst                   db $0C,$86,$86,$82,"Instructions",$0D,$0D,cCyan
                        db "Search the tower for the seven rings and"
                        db cCret
                        db "try not to get killed while doing it."
                        db cCret
                        db "Then find the key in the bottom level to"
                        db cCret
                        db "escape.",cCret,cCret

                        db "Drink the magic potions (walk over them)"
                        db cCret
                        db "to gain magic. This will automatically heal"
                        db cCret
                        db "wounds and power your weapons."
                        db cCret,cCret
                        db "You don't have to attack the things but"
                        db cCret
                        db "they'll show you no mercy either way."
                        db cCret,cCret
                        db "Use 'space' to align the screen often."
                        db cCret
                        db "Lightning '",ckLightning,"' repeat fires and Kickaha will"
                        db cCret
                        db "defend himself with the sword automatically,"
                        db cCret
                        db "but that won't help you much, so good luck..."
                        db cCret,cCret
                        db "Oh - blitz-halls? Fires automatically, and"
                        db cCret
                        db "your reflections can't hurt you. Have fun..."
                        db cCret,cCret,cMagenta
                        db "The source code is available from"
                        db cCret
                        db "www.desdes.com/products/oldfiles"
                        db 0


sHighA                  defb $0C,$86,$86,"Halls Of The Things",$0D,$0D
                        defb $86,$86,$83,$05,"Hall of Fame",$0D,$0D,$0D,$00

sHighB                  defb $0D,$0D,"  Press ",$90,$05,"SPACE ",$06
                        defb "to play or ",$05,"ENTER ",$06,"for options",$00

sAccept                 defb $0D,$0D,$85,"Please type in your entry.",$03,$00

; Text buffers, put these in the big-maze area.

        struct
                        org BigMaze
HSBuffer                ds 128
LineBuffer              defs 30                         ;

; Keyboard buffer

CapsShift               equ KeyCounts                   ;
KeyCounts               defs 36                         ;
SymbolShift             defs 4                          ;

LastScore               ds 2                            ;
AnEssFlag               ds 0                            ; A flag

        send

; Include the game code here

                        include "g.inc"                 ;

; Include the fancy loader

                        include "loader.inc"            ;

; We want to tell the emulator about the writeable sections

        mem_void $0000,$4000                    ; ROM

        mem_var $4000,$1B00                     ; Screen

        mem_var Front_Vars_Start,Front_Vars_End-Front_Vars_Start  ; Front-end variables

