; Fancy loader... This uses Zeus's MODE-4 to interleave code/data and load a data file.

if zeusver < 54 ; Older versions cannot generate these loaders
                zeusprint ""
                zeusprint "Sorry, but this version of Zeus is not able to generate MODE-4 loaders."
                zeusprint "Please visit www.desdes.com/products/oldfiles to download a more recent version"
                zeusprint ""
                        Zeus_PC = Start                 ; Without loaders we just run the game

else

;bCompressForTape        equ true                        ; If true Zeus compresses the data placed on tape.
bTestLoader             equ true                        ; If true Zeus doesn't emulate the game, but resets the machine
bTurboLoader            equ true                        ; If enabled, uses frequencies 150% higher than normal

; This is how many zero bits we want Zeus to insert into the bit stream to give us time to do things
; when the user code is called. Zeus adds two zeros, then this many, then a one.
DelayTimeInBits         equ 0                           ; We don't need any extra time to display a character

; Colours used during this custom loader

Col1                    equ 1                           ; One border colour
Col2                    equ 1                           ; Border xor bits

; With Mode 4 Zeus lets you set the timing for the custom loader block by setting values for the following symbols.
; If you don't set any values for these following symbols Zeus uses the standard Spectrum timing
; I'm changing the pilot-tone frequency here so it's twice the frequency exactly half-way between a zero and a one
; Then the loader averages the pilot tone period, halves it to get the mid point frequency, and uses that value to
; discriminate between zero and one... this way it tracks tape speed.
; Ideally we'd track it on every pulse, but that takes more code.

        if not enabled bTurboLoader
zx_customstartpause        equ 0                        ; Pause after loading the loader
zx_custompulselength_pilot equ 1283*2                   ; Pilot tone pulse length        CHANGED
zx_custompilotlength       equ 500                      ; Pilot tone pulse count         CHANGED
zx_custompulselength_sync1 equ 855                      ; Sync 1 pulse length            CHANGED
zx_custompulselength_sync2 equ 855                      ; Sync 2 pulse length            CHANGED
zx_custompulselength_zero  equ 855                      ; Zero bit pulse length
zx_custompulselength_one   equ 1710                     ; One bit pulse length
zx_customendpause          equ 0                        ; Pause after loading the whole application
        else
zx_customstartpause        equ 0                        ; Pause after loading the loader
zx_custompulselength_pilot equ 1710                     ; Pilot tone pulse length        CHANGED
zx_custompilotlength       equ 500                      ; Pilot tone pulse count         CHANGED
zx_custompulselength_sync1 equ 570                      ; Sync 1 pulse length            CHANGED
zx_custompulselength_sync2 equ 570                      ; Sync 2 pulse length            CHANGED
zx_custompulselength_zero  equ 570                      ; Zero bit pulse length
zx_custompulselength_one   equ 1140                     ; One bit pulse length
zx_customendpause          equ 0                        ; Pause after loading the whole application
        endif

; Guesstimate a time for the leader pulse discrimination
LeaderTooLow               equ zx_custompulselength_pilot/85

; The loader code

                        org BigMaze                     ; We'll put the loader where BigMaze is as that is not used during loading

OurLoader               proc                            ;

Start                   ld sp,SafeStack                 ; Use somewhere we know is safe

                        ld bc,$7FFD                     ; The memory port address
                        exx                             ; Into BC'

                        ld c,Col1                       ; The normal border colour

                        ; C will always be zero when we get here (from the LDIR)

LeaderReset             ld hl,0                         ; Zero the period acc and loop counter
                        ld d,l                          ;

                        call TimeEdge                   ; Time in A

LeaderLp                call TimeEdge                   ; Return a time less than 50

                        cp LeaderTooLow                 ; Normally Shorter than this is too short
                        jr c LeaderReset                ; Too short

                        ; This pulse is a valid leader pulse

                        add a,l                         ; Accumulate the period during the leader
                        ld l,a                          ;
                        adc a,h                         ;
                        sub l                           ;
                        ld h,a                          ;

                        dec d                           ; Inc the counter
                        jr nz LeaderLp                  ; Not enough yet

                        ; h now contains the average of the 256 leader pulse lengths
                        ; this is twice the mid-time

                        sra h                           ; /2
                        ld e,h                          ; Save the mid-bit time

                        ; Okay, we wait for the Sync pulse

LeaderValid             call TimeEdge                   ; Return a time less than 50

                        cp e                            ; Short enough to be the sync?
                        jr nc LeaderValid               ;

                        call TimeEdge                   ; Could be a sync

                        cp e                            ; Short enough to be the sync?
                        jr nc LeaderValid               ; No...

                        ; We've had the sync pulse

GetAddress              ld d,0                          ; Clear the CS
                        call ReadByte_AL                ; Get the address first
                        ld ixl,a                        ;
                        call ReadByte_AL                ; Addr High
                        ld ixh,a                        ;
                        call ReadByte_AL                ; Flag byte

                        ; Page or flag... if the top bit is set this is data and start the user code with it

                        or a                            ; Test the top bit
                        jp m HPR_GetCS                  ; It's set, this is data for the user code, check the CS

                        ; It's not data, assume it's page data

                        and %00 011 111                 ; Mask off the dangerous bits
                        exx                             ; Write the page register
                        out (c),a                       ; Remember we have BC' containing the page register address
                        exx                             ;

                        call ReadByte_AL                ; Data length
                        ld h,a                          ;

DataLoop                call ReadByte_AL                ; Data length
                        ld (ix),a                       ; Store it
                        inc ix                          ;

                        dec h                           ; Dec the data interleave count
                        jr nz DataLoop                  ; Another data, skip

                        ; We've read a complete block

                        call ReadByte_AL                ; CS

                        ld a,d                          ; CS = 0?
                        or a                            ;
                        jr z GetAddress                 ; Yes, loop

                        jr Start                        ; CS failed.

        ; This is the user byte after the flag byte.
        ; Normally this would be the CS for the user block, but if we have asked zeus to plant extra bytes this
        ; is when they need to be read. Then the CS after them.

HPR_GetCS               ld h,a                          ; Save the flag byte

                        call ReadByte_AL                ; The CS for the user block

                        ld a,d                          ; CS = 0?
                        or a                            ;
                        jp nz Start                     ; No, fail

                        ; User code here...

                        ; The user code goes here... we have 16-bit data in IX and 7-bit data in h (with bit 7 set)
                        ;
                        ; This code must either not alter BC' or set it back to 7FFD
                        ; It must save E, C
                        ; It may use A,B,D,HL,IX,IY

UserCode                ld a,h                          ; Get the 7-bit parameter. The 16 bit one is in IX
                        cp $FF                          ; End marker (added by Zeus)
        if enabled bCompressForTape
                        jp z $5B00                      ; Yes, start the expander
        else
                        jp z ::Start                    ; Yes, start the code
        endif

                        ld b,e                          ; Save E and C
                        push bc                         ;

                        ; We're using this as a character index
                        ; When it has finished it must jump back to Resync

                        and $7F                         ; Lose the top bit
                        ld l,a                          ;
                        ld h,0                          ;
                        ld de,hl                        ;
                        add hl,hl                       ; * 9
                        add hl,hl                       ;
                        add hl,hl                       ;
                        add hl,de                       ;
                        ld de,LoaderChars               ;
                        add hl,de                       ;

                        push ix                         ; Get the address
                        pop de                          ;

                        ld b,8                          ;
LoadCharLp              ld a,(hl)                       ;
                        inc hl                          ;
                        ld (de),a                       ;
                        inc d                           ;
                        djnz LoadCharLp                 ;

                        ; Now figure out the attribute address

                        ld a,ixh                        ;
                        rrca                            ;
                        rrca                            ;
                        rrca                            ;
                        and %000000 11                  ;
                        or $58                          ;
                        ld d,a                          ;

                        ld a,(hl)                       ;
                        ld (de),a                       ;

                        ; This code resynchronises with the data stream by looking for the next "1" bit to wander along

                        pop bc                          ; Restore C and E
                        ld e,b                          ;

Resync                  ld d,0                          ; Clear the CS

                        call TimeEdge                   ;

ResyncLp                call TimeEdge                   ;

                        cp e                            ;
                        jr c ResyncLp                   ; Wait while zero

                        call TimeEdge                   ; Lose the second half

                        jp GetAddress                   ; Get the next address

; Measure the period of a bit

TimeEdge2               call TimeEdge                   ; Miss a half-cycle

TimeEdge                ld b,8                          ; Clear the period

TimeEdgeDel             djnz TimeEdgeDel                ; Brief delay at start

TimeEdgeLp              ld a,$7F                        ; Read the port
                        in a,($FE)                      ;

                        inc b                           ; Period++
                        jp z Start                      ; Overflow

                        xor c                           ; Compare
                        and $40                         ; Check the input
                        jr nz TimeEdgeLp                ; Loop

                        ; The input has changed

                        ld a,c                          ; Flip the bit
                        xor (Col2 and %00000 111) or %11111 000 ; Toggle the bit and the border iff required
                        out ($FE),a                     ;
                        ld c,a                          ;

                        ld a,b                          ; Return the period

                        ret                             ;

; Read the next byte

ReadByte_AL             ld l,$01                        ; Sentinel bit

ReadByteLp              call TimeEdge2                  ; Miss one half cycle and time the next

                        cp e                            ; 0 or 1
                        ccf                             ; Flip it
                        rl l                            ; bit rotate

                        jp nc ReadByteLp                ;

                        ; L = Next byte

                        ld a,d                          ; Calc CS
                        xor l                           ;
                        ld d,a                          ;

                        ld a,l                          ; Return the result in A as well.
                        ret                             ; Done

; Loader characters

LoaderChars equ *
::zeuseditfontbase equ *                ; Tell Zeus this is a character set so we can use them to edit the screen

; Loader characters
LoaderChars equ *

; Char 00
  dg ...XX...
  dg ..XXXX..
  dg .XX..XX.
  dg XXX..XXX
  dg XXXXXXXX
  dg XXXXXXXX
  dg ..X..X..
  dg .XX..XX.
  db $43

; Char 01
  dg ...XX...
  dg ...XX...
  dg ..X.XX..
  dg ..XXX.XX
  dg .XX.XXX.
  dg .XXXX.X.
  dg XXX.XXXX
  dg XXXXXXXX
  db $43

; Char 02
  dg ...XX...
  dg ...XX...
  dg ..XX.X..
  dg XX.XXX..
  dg .XXX.XX.
  dg .X.XXXX.
  dg XXXX.XXX
  dg XXXXXXXX
  db $43

; Char 03
  dg ...XX...
  dg ..XXXX..
  dg .XXXXXX.
  dg XXXXXXXX
  dg XXXXXXXX
  dg XXXXXXXX
  dg ..X..X..
  dg .XX..XX.
  db $43

; Char 04
  dg ...XX...
  dg ..XXXX..
  dg .XXXXXX.
  dg XX.XX.XX
  dg XX.XX.XX
  dg XXXXXXXX
  dg ..X..X..
  dg .XX..XX.
  db $43

; Char 05
  dg ..XXXX..
  dg .XX..XX.
  dg XXX..XXX
  dg X.XXXX.X
  dg X.XXXX.X
  dg ..X..X..
  dg ..X..X..
  dg .XX..XX.
  db $43

; Char 06
  dg ........
  dg ..XXXX..
  dg .X....X.
  dg .X....X.
  dg .X....X.
  dg .X....X.
  dg ..XXXX..
  dg ........
  db $42

; Char 07
  dg ........
  dg ..XXXX..
  dg .X....X.
  dg .X....X.
  dg .X....X.
  dg .X....X.
  dg ..XXXX..
  dg ........
  db $41

; Char 08
  dg ........
  dg ..XXXX..
  dg .X....X.
  dg .X....X.
  dg .X....X.
  dg .X....X.
  dg ..XXXX..
  dg ........
  db $04

; Char 09
  dg ........
  dg ..XXXX..
  dg .X....X.
  dg .X....X.
  dg .X....X.
  dg .X....X.
  dg ..XXXX..
  dg ........
  db $45

; Char 0A
  dg ........
  dg ..XXXX..
  dg .X....X.
  dg .X....X.
  dg .X....X.
  dg .X....X.
  dg ..XXXX..
  dg ........
  db $06

; Char 0B
  dg ........
  dg ..XXXX..
  dg .X....X.
  dg .X....X.
  dg .X....X.
  dg .X....X.
  dg ..XXXX..
  dg ........
  db $07

::zeuseditfontcount equ (* - zeuseditfontbase) / 9;

Length                  equ * - Start                   ; Calc the length

                ; We put these bytes here so we don't load them...

                        ds 6                            ; Somewhere for the stack
SafeStack               equ *                           ;

                        pend                            ; End of our loader

; Now, this is the parameter data for the loader

; Loader parameters - this loader uses the available 16-bit parameter as a raw address, and the 7-bit parameter as a character index.

zx_customextradatacount equ 0                           ; We just have the normal three parameters, 16+7 bit(NYI?)

; Loader parameters

  org zeusrom(-1)
  dw $4000:db $00
  dw $4001:db $00
  dw $4002:db $00
  dw $4003:db $00
  dw $4004:db $00
  dw $4005:db $00
  dw $4006:db $00
  dw $4007:db $00
  dw $4008:db $00
  dw $4009:db $00
  dw $400A:db $00
  dw $400B:db $00
  dw $400C:db $00
  dw $400D:db $00
  dw $400E:db $00
  dw $400F:db $00
  dw $4010:db $00
  dw $4011:db $00
  dw $4012:db $00
  dw $4013:db $00
  dw $4014:db $00
  dw $4015:db $00
  dw $4016:db $00
  dw $4017:db $00
  dw $4018:db $00
  dw $4019:db $00
  dw $401A:db $00
  dw $401B:db $00
  dw $401C:db $00
  dw $401D:db $00
  dw $401E:db $00
  dw $401F:db $00
  dw $403F:db $00
  dw $405F:db $00
  dw $407F:db $00
  dw $409F:db $00
  dw $40BF:db $00
  dw $40DF:db $00
  dw $40FF:db $00
  dw $481F:db $00
  dw $483F:db $00
  dw $485F:db $00
  dw $487F:db $00
  dw $489F:db $00
  dw $48BF:db $00
  dw $48DF:db $00
  dw $48FF:db $00
  dw $501F:db $00
  dw $503F:db $00
  dw $505F:db $00
  dw $507F:db $00
  dw $509F:db $00
  dw $50BF:db $00
  dw $50DF:db $00
  dw $50FF:db $00
  dw $50FE:db $00
  dw $50FD:db $00
  dw $50FC:db $00
  dw $50FB:db $00
  dw $50FA:db $00
  dw $50F9:db $00
  dw $50F8:db $00
  dw $50F7:db $00
  dw $50F6:db $00
  dw $50F5:db $00
  dw $50F4:db $00
  dw $50F3:db $00
  dw $50F2:db $00
  dw $50F1:db $00
  dw $50F0:db $00
  dw $50EF:db $00
  dw $50EE:db $00
  dw $50ED:db $00
  dw $50EC:db $00
  dw $50EB:db $00
  dw $50EA:db $00
  dw $50E9:db $00
  dw $50E8:db $00
  dw $50E7:db $00
  dw $50E6:db $00
  dw $50E5:db $00
  dw $50E4:db $00
  dw $50E3:db $00
  dw $50E2:db $00
  dw $50E1:db $00
  dw $50E0:db $00
  dw $50C0:db $00
  dw $50A0:db $00
  dw $5080:db $00
  dw $5060:db $00
  dw $5040:db $00
  dw $5020:db $00
  dw $5000:db $00
  dw $48E0:db $00
  dw $48C0:db $00
  dw $48A0:db $00
  dw $4880:db $00
  dw $4860:db $00
  dw $4840:db $00
  dw $4820:db $00
  dw $4800:db $00
  dw $40E0:db $00
  dw $40C0:db $00
  dw $40A0:db $00
  dw $4080:db $00
  dw $4060:db $00
  dw $4040:db $00
  dw $4020:db $00
  dw $4021:db $01
  dw $4022:db $01
  dw $4023:db $01
  dw $4024:db $01
  dw $4025:db $01
  dw $4026:db $01
  dw $4027:db $01
  dw $4028:db $01
  dw $4029:db $01
  dw $402A:db $01
  dw $402B:db $01
  dw $402C:db $01
  dw $402D:db $01
  dw $402E:db $01
  dw $402F:db $01
  dw $4030:db $01
  dw $4031:db $01
  dw $4032:db $01
  dw $4033:db $01
  dw $4034:db $01
  dw $4035:db $01
  dw $4036:db $01
  dw $4037:db $01
  dw $4038:db $01
  dw $4039:db $01
  dw $403A:db $01
  dw $403B:db $01
  dw $403C:db $01
  dw $403D:db $01
  dw $403E:db $01
  dw $405E:db $01
  dw $407E:db $01
  dw $409E:db $01
  dw $40BE:db $01
  dw $40DE:db $01
  dw $40FE:db $01
  dw $481E:db $01
  dw $483E:db $01
  dw $485E:db $01
  dw $487E:db $01
  dw $489E:db $01
  dw $48BE:db $01
  dw $48DE:db $01
  dw $48FE:db $01
  dw $501E:db $01
  dw $503E:db $01
  dw $505E:db $01
  dw $507E:db $01
  dw $509E:db $01
  dw $50BE:db $01
  dw $50DE:db $01
  dw $50DD:db $01
  dw $50DC:db $01
  dw $50DB:db $01
  dw $50DA:db $01
  dw $50D9:db $01
  dw $50D8:db $01
  dw $50D7:db $01
  dw $50D6:db $01
  dw $50D5:db $01
  dw $50D4:db $01
  dw $50D3:db $01
  dw $50D2:db $01
  dw $50D1:db $01
  dw $50D0:db $01
  dw $50CF:db $01
  dw $50CE:db $01
  dw $50CD:db $01
  dw $50CC:db $01
  dw $50CB:db $01
  dw $50CA:db $01
  dw $50C9:db $01
  dw $50C8:db $01
  dw $50C7:db $01
  dw $50C6:db $01
  dw $50C5:db $01
  dw $50C4:db $01
  dw $50C3:db $01
  dw $50C2:db $01
  dw $50C1:db $01
  dw $50A1:db $01
  dw $5081:db $01
  dw $5061:db $01
  dw $5041:db $01
  dw $5021:db $01
  dw $5001:db $01
  dw $48E1:db $01
  dw $48C1:db $01
  dw $48A1:db $01
  dw $4881:db $01
  dw $4861:db $01
  dw $4841:db $01
  dw $4821:db $01
  dw $4801:db $01
  dw $40E1:db $01
  dw $40C1:db $01
  dw $40A1:db $01
  dw $4081:db $01
  dw $4061:db $01
  dw $4041:db $01
  dw $4042:db $02
  dw $4043:db $02
  dw $4044:db $02
  dw $4045:db $02
  dw $4046:db $02
  dw $4047:db $02
  dw $4048:db $02
  dw $4049:db $02
  dw $404A:db $02
  dw $404B:db $02
  dw $404C:db $02
  dw $404D:db $02
  dw $404E:db $02
  dw $404F:db $02
  dw $4050:db $02
  dw $4051:db $02
  dw $4052:db $02
  dw $4053:db $02
  dw $4054:db $02
  dw $4055:db $02
  dw $4056:db $02
  dw $4057:db $02
  dw $4058:db $02
  dw $4059:db $02
  dw $405A:db $02
  dw $405B:db $02
  dw $405C:db $02
  dw $405D:db $02
  dw $407D:db $02
  dw $409D:db $02
  dw $40BD:db $02
  dw $40DD:db $02
  dw $40FD:db $02
  dw $481D:db $02
  dw $483D:db $02
  dw $485D:db $02
  dw $487D:db $02
  dw $489D:db $02
  dw $48BD:db $02
  dw $48DD:db $02
  dw $48FD:db $02
  dw $501D:db $02
  dw $503D:db $02
  dw $505D:db $02
  dw $507D:db $02
  dw $509D:db $02
  dw $50BD:db $02
  dw $50BC:db $02
  dw $50BB:db $02
  dw $50BA:db $02
  dw $50B9:db $02
  dw $50B8:db $02
  dw $50B7:db $02
  dw $50B6:db $02
  dw $50B5:db $02
  dw $50B4:db $02
  dw $50B3:db $02
  dw $50B2:db $02
  dw $50B1:db $02
  dw $50B0:db $02
  dw $50AF:db $02
  dw $50AE:db $02
  dw $50AD:db $02
  dw $50AC:db $02
  dw $50AB:db $02
  dw $50AA:db $02
  dw $50A9:db $02
  dw $50A8:db $02
  dw $50A7:db $02
  dw $50A6:db $02
  dw $50A5:db $02
  dw $50A4:db $02
  dw $50A3:db $02
  dw $50A2:db $02
  dw $5082:db $02
  dw $5062:db $02
  dw $5042:db $02
  dw $5022:db $02
  dw $5002:db $02
  dw $48E2:db $02
  dw $48C2:db $02
  dw $48A2:db $02
  dw $4882:db $02
  dw $4862:db $02
  dw $4842:db $02
  dw $4822:db $02
  dw $4802:db $02
  dw $40E2:db $02
  dw $40C2:db $02
  dw $40A2:db $02
  dw $4082:db $02
  dw $4062:db $02
  dw $4063:db $03
  dw $4064:db $03
  dw $4065:db $03
  dw $4066:db $03
  dw $4067:db $03
  dw $4068:db $03
  dw $4069:db $03
  dw $406A:db $03
  dw $406B:db $03
  dw $406C:db $03
  dw $406D:db $03
  dw $406E:db $03
  dw $406F:db $03
  dw $4070:db $03
  dw $4071:db $03
  dw $4072:db $03
  dw $4073:db $03
  dw $4074:db $03
  dw $4075:db $03
  dw $4076:db $03
  dw $4077:db $03
  dw $4078:db $03
  dw $4079:db $03
  dw $407A:db $03
  dw $407B:db $03
  dw $407C:db $03
  dw $409C:db $03
  dw $40BC:db $03
  dw $40DC:db $03
  dw $40FC:db $03
  dw $481C:db $03
  dw $483C:db $03
  dw $485C:db $03
  dw $487C:db $03
  dw $489C:db $03
  dw $48BC:db $03
  dw $48DC:db $03
  dw $48FC:db $03
  dw $501C:db $03
  dw $503C:db $03
  dw $505C:db $03
  dw $507C:db $03
  dw $509C:db $03
  dw $509B:db $03
  dw $509A:db $03
  dw $5099:db $03
  dw $5098:db $03
  dw $5097:db $03
  dw $5096:db $03
  dw $5095:db $03
  dw $5094:db $03
  dw $5093:db $03
  dw $5092:db $03
  dw $5091:db $03
  dw $5090:db $03
  dw $508F:db $03
  dw $508E:db $03
  dw $508D:db $03
  dw $508C:db $03
  dw $508B:db $03
  dw $508A:db $03
  dw $5089:db $03
  dw $5088:db $03
  dw $5087:db $03
  dw $5086:db $03
  dw $5085:db $03
  dw $5084:db $03
  dw $5083:db $03
  dw $5063:db $03
  dw $5043:db $03
  dw $5023:db $03
  dw $5003:db $03
  dw $48E3:db $03
  dw $48C3:db $03
  dw $48A3:db $03
  dw $4883:db $03
  dw $4863:db $03
  dw $4843:db $03
  dw $4823:db $03
  dw $4803:db $03
  dw $40E3:db $03
  dw $40C3:db $03
  dw $40A3:db $03
  dw $4083:db $03
  dw $4084:db $04
  dw $4085:db $04
  dw $4086:db $04
  dw $4087:db $04
  dw $4088:db $04
  dw $4089:db $04
  dw $408A:db $04
  dw $408B:db $04
  dw $408C:db $04
  dw $408D:db $04
  dw $408E:db $04
  dw $408F:db $04
  dw $4090:db $04
  dw $4091:db $04
  dw $4092:db $04
  dw $4093:db $04
  dw $4094:db $04
  dw $4095:db $04
  dw $4096:db $04
  dw $4097:db $04
  dw $4098:db $04
  dw $4099:db $04
  dw $409A:db $04
  dw $409B:db $04
  dw $40BB:db $04
  dw $40DB:db $04
  dw $40FB:db $04
  dw $481B:db $04
  dw $483B:db $04
  dw $485B:db $04
  dw $487B:db $04
  dw $489B:db $04
  dw $48BB:db $04
  dw $48DB:db $04
  dw $48FB:db $04
  dw $501B:db $04
  dw $503B:db $04
  dw $505B:db $04
  dw $507B:db $04
  dw $507A:db $04
  dw $5079:db $04
  dw $5078:db $04
  dw $5077:db $04
  dw $5076:db $04
  dw $5075:db $04
  dw $5074:db $04
  dw $5073:db $04
  dw $5072:db $04
  dw $5071:db $04
  dw $5070:db $04
  dw $506F:db $04
  dw $506E:db $04
  dw $506D:db $04
  dw $506C:db $04
  dw $506B:db $04
  dw $506A:db $04
  dw $5069:db $04
  dw $5068:db $04
  dw $5067:db $04
  dw $5066:db $04
  dw $5065:db $04
  dw $5064:db $04
  dw $5044:db $04
  dw $5024:db $04
  dw $5004:db $04
  dw $48E4:db $04
  dw $48C4:db $04
  dw $48A4:db $04
  dw $4884:db $04
  dw $4864:db $04
  dw $4844:db $04
  dw $4824:db $04
  dw $4804:db $04
  dw $40E4:db $04
  dw $40C4:db $04
  dw $40A4:db $04
  dw $40A5:db $05
  dw $40A6:db $05
  dw $40A7:db $05
  dw $40A8:db $05
  dw $40A9:db $05
  dw $40AA:db $05
  dw $40AB:db $05
  dw $40AC:db $05
  dw $40AD:db $05
  dw $40AE:db $05
  dw $40AF:db $05
  dw $40B0:db $05
  dw $40B1:db $05
  dw $40B2:db $05
  dw $40B3:db $05
  dw $40B4:db $05
  dw $40B5:db $05
  dw $40B6:db $05
  dw $40B7:db $05
  dw $40B8:db $05
  dw $40B9:db $05
  dw $40BA:db $05
  dw $40DA:db $05
  dw $40FA:db $05
  dw $481A:db $05
  dw $483A:db $05
  dw $485A:db $05
  dw $487A:db $05
  dw $489A:db $05
  dw $48BA:db $05
  dw $48DA:db $05
  dw $48FA:db $05
  dw $501A:db $05
  dw $503A:db $05
  dw $505A:db $05
  dw $5059:db $05
  dw $5058:db $05
  dw $5057:db $05
  dw $5056:db $05
  dw $5055:db $05
  dw $5054:db $05
  dw $5053:db $05
  dw $5052:db $05
  dw $5051:db $05
  dw $5050:db $05
  dw $504F:db $05
  dw $504E:db $05
  dw $504D:db $05
  dw $504C:db $05
  dw $504B:db $05
  dw $504A:db $05
  dw $5049:db $05
  dw $5048:db $05
  dw $5047:db $05
  dw $5046:db $05
  dw $5045:db $05
  dw $5025:db $05
  dw $5005:db $05
  dw $48E5:db $05
  dw $48C5:db $05
  dw $48A5:db $05
  dw $4885:db $05
  dw $4865:db $05
  dw $4845:db $05
  dw $4825:db $05
  dw $4805:db $05
  dw $40E5:db $05
  dw $40C5:db $05
  dw $40C6:db $06
  dw $40C7:db $06
  dw $40C8:db $06
  dw $40C9:db $06
  dw $40CA:db $06
  dw $40CB:db $06
  dw $40CC:db $06
  dw $40CD:db $06
  dw $40CE:db $06
  dw $40CF:db $06
  dw $40D0:db $06
  dw $40D1:db $06
  dw $40D2:db $06
  dw $40D3:db $06
  dw $40D4:db $06
  dw $40D5:db $06
  dw $40D6:db $06
  dw $40D7:db $06
  dw $40D8:db $06
  dw $40D9:db $06
  dw $40F9:db $06
  dw $4819:db $06
  dw $4839:db $06
  dw $4859:db $06
  dw $4879:db $06
  dw $4899:db $06
  dw $48B9:db $06
  dw $48D9:db $06
  dw $48F9:db $06
  dw $5019:db $06
  dw $5039:db $06
  dw $5038:db $06
  dw $5037:db $06
  dw $5036:db $06
  dw $5035:db $06
  dw $5034:db $06
  dw $5033:db $06
  dw $5032:db $06
  dw $5031:db $06
  dw $5030:db $06
  dw $502F:db $06
  dw $502E:db $06
  dw $502D:db $06
  dw $502C:db $06
  dw $502B:db $06
  dw $502A:db $06
  dw $5029:db $06
  dw $5028:db $06
  dw $5027:db $06
  dw $5026:db $06
  dw $5006:db $06
  dw $48E6:db $06
  dw $48C6:db $06
  dw $48A6:db $06
  dw $4886:db $06
  dw $4866:db $06
  dw $4846:db $06
  dw $4826:db $06
  dw $4806:db $06
  dw $40E6:db $06
  dw $40E7:db $07
  dw $40E8:db $07
  dw $40E9:db $07
  dw $40EA:db $07
  dw $40EB:db $07
  dw $40EC:db $07
  dw $40ED:db $07
  dw $40EE:db $07
  dw $40EF:db $07
  dw $40F0:db $07
  dw $40F1:db $07
  dw $40F2:db $07
  dw $40F3:db $07
  dw $40F4:db $07
  dw $40F5:db $07
  dw $40F6:db $07
  dw $40F7:db $07
  dw $40F8:db $07
  dw $4818:db $07
  dw $4838:db $07
  dw $4858:db $07
  dw $4878:db $07
  dw $4898:db $07
  dw $48B8:db $07
  dw $48D8:db $07
  dw $48F8:db $07
  dw $5018:db $07
  dw $5017:db $07
  dw $5016:db $07
  dw $5015:db $07
  dw $5014:db $07
  dw $5013:db $07
  dw $5012:db $07
  dw $5011:db $07
  dw $5010:db $07
  dw $500F:db $07
  dw $500E:db $07
  dw $500D:db $07
  dw $500C:db $07
  dw $500B:db $07
  dw $500A:db $07
  dw $5009:db $07
  dw $5008:db $07
  dw $5007:db $07
  dw $48E7:db $07
  dw $48C7:db $07
  dw $48A7:db $07
  dw $4887:db $07
  dw $4867:db $07
  dw $4847:db $07
  dw $4827:db $07
  dw $4807:db $07
  dw $4808:db $08
  dw $4809:db $08
  dw $480A:db $08
  dw $480B:db $08
  dw $480C:db $08
  dw $480D:db $08
  dw $480E:db $08
  dw $480F:db $08
  dw $4810:db $08
  dw $4811:db $08
  dw $4812:db $08
  dw $4813:db $08
  dw $4814:db $08
  dw $4815:db $08
  dw $4816:db $08
  dw $4817:db $08
  dw $4837:db $08
  dw $4857:db $08
  dw $4877:db $08
  dw $4897:db $08
  dw $48B7:db $08
  dw $48D7:db $08
  dw $48F7:db $08
  dw $48F6:db $08
  dw $48F5:db $08
  dw $48F4:db $08
  dw $48F3:db $08
  dw $48F2:db $08
  dw $48F1:db $08
  dw $48F0:db $08
  dw $48EF:db $08
  dw $48EE:db $08
  dw $48ED:db $08
  dw $48EC:db $08
  dw $48EB:db $08
  dw $48EA:db $08
  dw $48E9:db $08
  dw $48E8:db $08
  dw $48C8:db $08
  dw $48A8:db $08
  dw $4888:db $08
  dw $4868:db $08
  dw $4848:db $08
  dw $4828:db $08
  dw $4829:db $09
  dw $482A:db $09
  dw $482B:db $09
  dw $482C:db $09
  dw $482D:db $09
  dw $482E:db $09
  dw $482F:db $09
  dw $4830:db $09
  dw $4831:db $09
  dw $4832:db $09
  dw $4833:db $09
  dw $4834:db $09
  dw $4835:db $09
  dw $4836:db $09
  dw $4856:db $09
  dw $4876:db $09
  dw $4896:db $09
  dw $48B6:db $09
  dw $48D6:db $09
  dw $48D5:db $09
  dw $48D4:db $09
  dw $48D3:db $09
  dw $48D2:db $09
  dw $48D1:db $09
  dw $48D0:db $09
  dw $48CF:db $09
  dw $48CE:db $09
  dw $48CD:db $09
  dw $48CC:db $09
  dw $48CB:db $09
  dw $48CA:db $09
  dw $48C9:db $09
  dw $48A9:db $09
  dw $4889:db $09
  dw $4869:db $09
  dw $4849:db $09
  dw $484A:db $0A
  dw $484B:db $0A
  dw $484C:db $0A
  dw $484D:db $0A
  dw $484E:db $0A
  dw $484F:db $0A
  dw $4850:db $0A
  dw $4851:db $0A
  dw $4852:db $0A
  dw $4853:db $0A
  dw $4854:db $0A
  dw $4855:db $0A
  dw $4875:db $0A
  dw $4895:db $0A
  dw $48B5:db $0A
  dw $48B4:db $0A
  dw $48B3:db $0A
  dw $48B2:db $0A
  dw $48B1:db $0A
  dw $48B0:db $0A
  dw $48AF:db $0A
  dw $48AE:db $0A
  dw $48AD:db $0A
  dw $48AC:db $0A
  dw $48AB:db $0A
  dw $48AA:db $0A
  dw $488A:db $0A
  dw $486A:db $0A
  dw $486B:db $0B
  dw $486C:db $0B
  dw $486D:db $0B
  dw $486E:db $0B
  dw $486F:db $0B
  dw $4870:db $0B
  dw $4871:db $0B
  dw $4872:db $0B
  dw $4873:db $0B
  dw $4874:db $0B
  dw $4894:db $0B
  dw $4893:db $0B
  dw $4892:db $0B
  dw $4891:db $0B
  dw $4890:db $0B
  dw $488F:db $0B
  dw $488E:db $0B
  dw $488D:db $0B
  dw $488C:db $0B
  dw $488B:db $0B

SegmentCnt equ 768

; Generate a tape with this loader, we want SegmentCnt "blocks"

                        ; Get the loader code loading screen

                        import_bin "HallsEmpty.scr",$4000

        if enabled bCompressForTape
                        ; First we compress the game code.
                        zeuscompress $5B00,Ye_End,Start,$5B00,1 ; Compress the game code into a self-extracting lump of bytes.

                        ; Now we load the loader and the first loading screen
                        output_tzx "hott.tzx","CheeseCake","Build with Zeus",OurLoader.Start,OurLoader.Length,4,OurLoader.Start,$06000101,SegmentCnt,DelayTimeInBits

                        ; Now we have to provide the data areas that will be loaded by the fancy loader
                        ; This is just the code block produced by the compressor.
                        output_tzx_block "hott.tzx",$5B00,*-$5B00       ; The code generated by the compressor
        else
                        ; Now we load the loader and the first loading screen
                        output_tzx "hott.tzx","CheeseCake","Build with Zeus",OurLoader.Start,OurLoader.Length,4,OurLoader.Start,$06000101,SegmentCnt,DelayTimeInBits

                        ; Now we have to provide all the data areas that will be loaded by the fancy loader
                        output_tzx_block "hott.tzx",TapeSaveRegion1Start,TapeSaveRegion1Length
                        output_tzx_block "hott.tzx",TapeSaveRegion2Start,TapeSaveRegion2Length
                        output_tzx_block "hott.tzx",TapeSaveRegion3Start,TapeSaveRegion3Length
                        output_tzx_block "hott.tzx",TapeSaveRegion4Start,TapeSaveRegion4Length
        endif

; And much the same again to generate a "*.tap" file.

        if enabled bTurboTiming
          zeusprint "WARNING! TAP files do not support turbo loading... not generating 'hott.tap'"
        else
         if enabled bCompressForTape
                        ; First we compress the game code.
                        zeuscompress $5B00,Ye_End,Start,$5B00,1 ; Compress the game code into a self-extracting lump of bytes.

                        ; Now we load the loader and the first loading screen
                        output_tap "hott.tap","CheeseCake","Build with Zeus",OurLoader.Start,OurLoader.Length,4,OurLoader.Start,$06000101,SegmentCnt,DelayTimeInBits

                        ; Now we have to provide the data areas that will be loaded by the fancy loader
                        ; This is just the code block produced by the compressor.
                        output_tap_block "hott.tap",$5B00,*-$5B00       ; The code generated by the compressor
         else
                        ; Now we load the loader and the first loading screen
                        output_tap "hott.tap","CheeseCake","Build with Zeus",OurLoader.Start,OurLoader.Length,4,OurLoader.Start,$06000101,SegmentCnt,DelayTimeInBits

                        ; Now we have to provide all the data areas that will be loaded by the fancy loader
                        output_tap_block "hott.tap",TapeSaveRegion1Start,TapeSaveRegion1Length
                        output_tap_block "hott.tap",TapeSaveRegion2Start,TapeSaveRegion2Length
                        output_tap_block "hott.tap",TapeSaveRegion3Start,TapeSaveRegion3Length
                        output_tap_block "hott.tap",TapeSaveRegion4Start,TapeSaveRegion4Length
          endif
        endif

; Force the emulated PC to zero so we start the "OS" and can loaad the tape

        if bTestLoader
          Zeus_PC = 0;
        else
          Zeus_PC = Start;
        endif

endif
