/*
This file shows examples of the additions made to zeus-ish in v2.23..v2.50

If you want more sustantial files they're available at www.desdes.com


IMPORT_BIN
----------

Version 2.32 improved the "import_bin" pseudo-op to support loading parts of binary files.
    (e.g. import_bin "filepath", memory address, length to read, offset into file)
    It also now supports loading to default PC address
    (e.g. import_bin "filepath")


TEXT_TIDY
---------

Version 2.33 added a "TextTidy" option to the edit menu. Just does some simple text reformatting.
It usually improves the look of badly formatted source code, but it's easily fooled so I suggest
comparing the code generated before and after use to be sure it hasn't altered the source's meaning.


AUTOMATIC BACKUP
----------------

Version 2.34 added backup file options. You can configure Zeus to make backup files before every
file save operation. You can select the number of generations of backups to create and if they are
created in a single directory or in multiple directories.


SETRADIX
--------

Version 2.40 changed the way Zeus handles numbers with leading zeros. Prior to version 2.40 they
were assumed to be octal numbers, so that 010 was equal to eight.

From v2.40 onward Zeus allows the user to select the radix to be used with leading zero numbers
eg.

                SETRADIX 16

Fred            = 010000 ; Fred is set to 65536

                SETRADIX 2

Fred            = 010000 ; Fred is set to 16

Now, note that there are potential problems with backwards compatibility when using SETRADIX.
These are caused by Zeus's support for post-fix binary and hex numbers, ie, numeric digits
followed by a 'B' or 'H'

Consider the following code:

                SETRADIX 16

Fred            = 010B  ; Is this "010" in binary or 10B in hex?

Well, what Zeus does is as follows: if none of the sources have a SETRADIX command then it
interprets numbers using the post-fix operators, so 010B would be equal to 2 (010 in binary)
BUT if there has been a SETRADIX command then Zeus no longer allows post-fix operators, so
010B would be interpreted using the current radix.


ASSEMBLY LISTING
----------------

Version 2.41 added an option to show the binary generated by Zeus in the editor. To do this
you must be using the Syntax Highlighting editor.

To show the assembled code right-click on the editor window and select Toggle Code, this
will expand the editor margin to include a field to contain the output data.

Then assemble the source and information should appear in this margin. This contains the
address and upto 8 bytes of data generated by the source line.


BINARY COMPARISON
-----------------

Version 2.50 added an option to compare the code generated by Zeus with an existing binary,
this is useful when reverse engineering existing code or converting sources from other assemblers
for which the binary code is avaiable.

NOTE - the Spectaculator spectrum emulator can export raw binary files, these are what you
need for the comparison.

To use this you need only tell Zeus what binary file it is to compare the current project
with. I have provided a sample binary for the source code included in this example source,
except that some of the values in the binary do not match the values in the source.

I have provided an example below. To use the example please make sure the file test_v250.bin
is in the directory c:\zeustest or alter the source below to point at this file.
After you have assembled this file make sure the output data field is visible by right-clicking
on the editor window and selecting "Toggle Code" if it is not.

If this worked you should see that the editor window is now double-line spaced.

You should now be seeing the output code in the margin of the editor. If you scroll up and down
you should see the binary Zeus has generated in blue and where this code is different to the
binary the code in the binary will be shown in red below it. The editor report window should
also have the text "3 source lines generate bytes that differ" shown as well.

Note -  a convenient way to find lines that differ is to point the mouse at the
data field in the margin and use the scroll-wheel. This will scroll up and down between
lines where the source and binary code differ. Hold down CTRL while scrolling and this
+will also move between the files in multi-file projects.

The three differences here are changes in constants, as shown in the source comments.

If you comment the "import_comparison" line out again and assemble the source the
editor should return to single-spaced lines.

*/

; Example source

; Uncomment this next line to have Zeus compare the source with an existing binary file

;                import_comparison "test_250.bin",$8000


; Print control codes

cDel            equ $08                ;
cCret           equ $0D                ;
cCls            equ $0C                ;
cClr            equ $1B                ;
cHome           equ $0E                ;

; Print colour select characters

cBlack          equ $00                ;
cBlue           equ $01                ;
cRed            equ $02                ;
cMagenta        equ $03                ;
cGreen          equ $04                ;
cCyan           equ $05                ;
cYellow         equ $06                ;
cWhite          equ $07                ;

                output_szx "test_250.szx",0,Start       ; Generate code to test

                org $8000               ; Somewhere safe

Start           di                      ; Make sure we're not interrupted
                ld sp,0                 ; Set the stack to the top of memory

                ld a,cCls               ; Clear the screen
                call Print              ;

                ld hl,#0                ; Load a constant

                ld a,cRed               ; Show the first in red - WAS blue
                call Print              ;

                call PrintHex_HL        ; Show the value

                call Print_Sp           ; " "

                call PrintStrFollow     ; Print the string following this call
                noflow                  ; Turn the warning off
                db cCyan," Hello there!"
                db cCret,"exciting, isn't it...",0

                ; We're done...

                halt                    ; Stop all the excitement

; Print the zero-terminated string pointed to by HL

PrintStr        proc                    ;

                push hl,af              ;

Loop            ld a,(hl)               ;
                inc hl                  ;
                or a                    ;
                jr z,Exit               ;

                call Print              ;
                jr Loop                 ;

Exit            pop af,hl               ;
                retp                    ; (RET)

; Print the string following the call in memory

PrintStrFollow  proc                    ;

                ex (sp),hl              ;
                push af                 ;

Loop            ld a,(hl)               ;
                inc hl                  ;
                or a                    ;
                jr z,Exit               ;

                call Print              ;

                jr Loop                 ;

Exit            pop af                  ;
                ex (sp),hl              ;

                retp                    ; (RET)

; Print the contents of HL in hex

PrintHex_HL     ld a,h                  ;
                call PrintHex_A         ;

                ld a,l                  ;

                ; Print the contents of A in hex

PrintHex_A      call PH2                ;

PH2             rrca                    ;
                rrca                    ;
                rrca                    ;
                rrca                    ;

                push af                 ;

                and $0F                 ;

                add a,"0"               ;
                cp "9"+1                ;
                jr c,PH3                ;

                add a,"A"-("9"+1)       ;

PH3             call Print              ;

                pop af                  ;
                ret                     ;

; Print a space

Print_Sp        push af                 ; Save

                ld a,' '                ; Print a space
                call Print              ;

                pop af                  ; Done
                ret                     ;

; A print character routine. Proportionally spaced characters

Print           push ix,hl,de,bc,af     ; Save them

                cp cWhite+1             ;
                jr c,PrintColour        ;

                cp cCls                 ;
                jr z,PrintCls           ;

                cp cCret                ;
                jr z,PrintCret          ;

                cp cHome                ;
                jr z,PrintHome          ;

                ld h,PrintWidths/256    ;
                ld l,a                  ;

                ld a,(pCurX)            ;
                add a,(hl)              ;
                call c,PrintCret        ;

                call PrintOutChar       ;

                ld a,(pCurX)            ;
                add a,(hl)              ;
                ld (pCurX),a            ;

PrintExit       pop af,bc,de,hl,ix      ;
                ret                     ;

; Set a colour

PrintColour     cp 4                    ; Above green don't make them bright
                jr nc,PrintCol1         ;

                or $40                  ; Bright

PrintCol1       ld (CurCol),a           ;
                jr PrintExit            ;

; Carriage return

PrintCret       call PrintDoCret        ;
                jr PrintExit            ;

; Clear screen - this clears it slowly with a fade effect

PrintCls        ld e,$00                ;
                ld hl,$5800             ;
                ld bc,$02FF             ;

pC1             ld a,(hl)               ;
                and $07                 ;
                jr z,pC2                ;

                set 0,e                 ;
                dec (hl)                ;
pC2             cpi                     ;
                jp pe,pC1               ;

                call Delay              ;

                bit 0,e                 ;
                jr nz,PrintCls          ;

                ld hl,$4000             ;
                ld de,$4001             ;
                ld bc,$1800             ;

                xor a                   ;
                ld (hl),a               ;

                ldir                    ;

                ld a,(CurCol)           ;
                ld (hl),a               ;

                ld bc,$02FF             ;

                ldir                    ;

                xor a                   ;
                out ($FE),a             ;

PrintHome       xor a                   ;
                ld (pCurX),a            ;
                ld (ppCurY),a           ;

                ld a,cWhite             ;
                ld (CurCol),a           ;

                ld hl,$4000             ;
                ld (pCurY),hl           ;

                jr PrintExit            ;

; A suitable delay for the fade

Delay           push bc                 ;
                ld bc,1000              ;
Del1            cpi                     ;
                dec hl                  ;
                jp pe,Del1              ;
                pop bc                  ;
                ret                     ;

; Do the carriage return

PrintDoCret     ld a,(ppCurY)           ;
                inc a                   ;

PrintSetUpY     ld (ppCurY),a           ;

                and $1F                 ;

                push hl                 ;

                ld l,a                  ;
                ld h,$00                ;

                add hl,hl               ;

                ld de,YTable            ;
                add hl,de               ;

                ld a,(hl)               ;
                inc hl                  ;
                ld h,(hl)               ;
                ld l,a                  ;

                ld (pCurY),hl           ;

                xor a                   ;
                ld (pCurX),a            ;

                pop hl                  ;
                ret                     ;

; Output the character

PrintOutChar    bit 7,l                 ; characters $00..$7F only
                ret nz                  ;

                push hl                 ;

                ld h,0                  ;

                add hl,hl               ;
                add hl,hl               ;
                add hl,hl               ;

                ld de,PrintChars-256    ;
                add hl,de               ;

                push hl                 ;
                pop ix                  ;

                ld a,(pCurX)            ;

                rrca                    ;
                rrca                    ;
                rrca                    ;
                and $1F                 ;

                ld de,(pCurY)           ;

                or e                    ;
                ld e,a                  ;

                push de                 ;

                ld a,d                  ;
                rrca                    ;
                rrca                    ;
                rrca                    ;
                and $03                 ;
                or $58                  ;
                ld d,a                  ;

                ld a,(CurCol)           ;
                ld (de),a               ;
                inc de                  ;
                ld (de),a               ;

                pop de                  ;

                ld a,(pCurX)            ;
                and $07                 ;
                ld c,a                  ;

                ld b,$08                ;

OutCharL        ld h,(ix)               ;
                ld l,$00                ;

                ld a,c                  ;
                or a                    ;
                jr z,OCL1               ;

OCL2            srl h                   ;
                rr l                    ;
                dec a                   ;
                jr nz,OCL2              ;

OCL1            ld a,(de)               ;
                xor h                   ;
                ld (de),a               ;

                inc e                   ;

                ld a,(de)               ;
                xor l                   ;
                ld (de),a               ;

                dec e                   ;

                inc ix                  ;
                inc d                   ;

                djnz OutCharL           ;

                pop hl                  ;
                ret                     ;

; Screen addresses for the character rows

YTable          defw $4000,$4020,$4040,$4060    ;
                defw $4080,$40A0,$40C0,$40E0    ;
                defw $4800,$4820,$4840,$4860    ;
                defw $4880,$48A0,$48C0,$48E0    ;
                defw $5000,$5020,$5040,$5060    ;
                defw $5080,$50A0,$50C0,$50E0    ;
                defw 0,0,0,0,0,0,0,0            ; So running off the end isn't fatal

; Character set graphics

                align 256       ; Put these on page boundaries

PrintChars      dg ........
                dg ........
                dg ........
                dg ........
                dg ........
                dg ........
                dg ........
                dg ........

                dg X.......
                dg X.......
                dg X.......
                dg X.......
                dg X.......
                dg ........
                dg X.......
                dg ........

                dg X.X.....
                dg X.X.....
                dg X.X.....
                dg ........
                dg ........
                dg ........
                dg ........
                dg ........

                dg .X.X....
                dg .X.X....
                dg XXXXX...
                dg .X.X....
                dg XXXXX...
                dg .X.X....
                dg .X.X....
                dg ........

                dg ..X.....
                dg .XXXX...
                dg X.X.....
                dg .XXX....
                dg ..X.X...
                dg XXXX....
                dg ..X.....
                dg ........

                dg XX......
                dg XX..X...
                dg ...X....
                dg ..X.....
                dg .X......
                dg X..XX...
                dg ...XX...
                dg ........

                dg .X......
                dg X.X.....
                dg X.X.....
                dg .X......
                dg X.X.X...
                dg X..X....
                dg .XX.X...
                dg ........

                dg X.......
                dg X.......
                dg X.......
                dg ........
                dg ........
                dg ........
                dg ........
                dg ........

                dg ..X.....
                dg .X......
                dg X.......
                dg X.......
                dg X.......
                dg .X......
                dg ..X.....
                dg ........

                dg X.......
                dg .X......
                dg ..X.....
                dg ..X.....
                dg ..X.....
                dg .X......
                dg X.......
                dg ........

                dg ..X.....
                dg X.X.X...
                dg .XXX....
                dg ..X.....
                dg .XXX....
                dg X.X.X...
                dg ..X.....
                dg ........

                dg ........
                dg ..X.....
                dg ..X.....
                dg XXXXX...
                dg ..X.....
                dg ..X.....
                dg ........
                dg ........

                dg ........
                dg ........
                dg ........
                dg ........
                dg .X......
                dg .X......
                dg X.......
                dg ........

                dg ........
                dg ........
                dg ........
                dg XXXXX...
                dg ........
                dg ........
                dg ........
                dg ........

                dg ........
                dg ........
                dg ........
                dg ........
                dg ........
                dg ........
                dg X.......
                dg ........

                dg ........
                dg ....X...
                dg ...X....
                dg ..X.....
                dg .X......
                dg X.......
                dg ........
                dg ........

                dg .XXX....
                dg X...X...
                dg X..XX...
                dg X.X.X...
                dg XX..X...
                dg X...X...
                dg .XXX....
                dg ........

                dg ..X.....
                dg .XX.....
                dg ..X.....
                dg ..X.....
                dg ..X.....
                dg ..X.....
                dg .XXX....
                dg ........

                dg .XXX....
                dg X...X...
                dg ....X...
                dg ..XX....
                dg .X......
                dg X.......
                dg XXXXX...
                dg ........

                dg XXXXX...
                dg ....X...
                dg ...X....
                dg ..XX....
                dg ....X...
                dg X...X...
                dg .XXX....
                dg ........

                dg ...X....
                dg ..XX....
                dg .X.X....
                dg X..X....
                dg XXXXX...
                dg ...X....
                dg ...X....
                dg ........

                dg XXXXX...
                dg X.......
                dg XXXX....
                dg ....X...
                dg ....X...
                dg X...X...
                dg .XXX....
                dg ........

                dg ..XXX...
                dg .X......
                dg X.......
                dg XXXX....
                dg X...X...
                dg X...X...
                dg .XXX....
                dg ........

                dg XXXXX...
                dg ....X...
                dg ...X....
                dg ..X.....
                dg .X......
                dg .X......
                dg .X......
                dg ........

                dg .XXX....
                dg X...X...
                dg X...X...
                dg .XXX....
                dg X...X...
                dg X...X...
                dg .XXX....
                dg ........

                dg .XXX....
                dg X...X...
                dg X...X...
                dg .XXXX...
                dg ....X...
                dg ...X....
                dg XXX.....
                dg ........

                dg ........
                dg ........
                dg X.......
                dg ........
                dg X.......
                dg ........
                dg ........
                dg ........

                dg ........
                dg ........
                dg .X......
                dg ........
                dg .X......
                dg .X......
                dg X.......
                dg ........

                dg ...X....
                dg ..X.....
                dg .X......
                dg X.......
                dg .X......
                dg ..X.....
                dg ...X....
                dg ........

                dg ........
                dg ........
                dg XXXXX...
                dg ........
                dg XXXXX...
                dg ........
                dg ........
                dg ........

                dg X.......
                dg .X......
                dg ..X.....
                dg ...X....
                dg ..X.....
                dg .X......
                dg X.......
                dg ........

                dg .XXX....
                dg X...X...
                dg ...X....
                dg ..X.....
                dg ..X.....
                dg ........
                dg ..X.....
                dg ........

                dg .XXX....
                dg X...X...
                dg X.X.X...
                dg X.XXX...
                dg X.XX....
                dg X.......
                dg .XXXX...
                dg ........

                dg ..X.....
                dg .X.X....
                dg X...X...
                dg X...X...
                dg XXXXX...
                dg X...X...
                dg X...X...
                dg ........

                dg XXXX....
                dg X...X...
                dg X...X...
                dg XXXX....
                dg X...X...
                dg X...X...
                dg XXXX....
                dg ........

                dg .XXX....
                dg X...X...
                dg X.......
                dg X.......
                dg X.......
                dg X...X...
                dg .XXX....
                dg ........

                dg XXXX....
                dg X...X...
                dg X...X...
                dg X...X...
                dg X...X...
                dg X...X...
                dg XXXX....
                dg ........

                dg XXXXX...
                dg X.......
                dg X.......
                dg XXXX....
                dg X.......
                dg X.......
                dg XXXXX...
                dg ........

                dg XXXXX...
                dg X.......
                dg X.......
                dg XXXX....
                dg X.......
                dg X.......
                dg X.......
                dg ........

                dg .XXXX...
                dg X.......
                dg X.......
                dg X.......
                dg X..XX...
                dg X...X...
                dg .XXXX...
                dg ........

                dg X...X...
                dg X...X...
                dg X...X...
                dg XXXXX...
                dg X...X...
                dg X...X...
                dg X...X...
                dg ........

                dg XXX.....
                dg .X......
                dg .X......
                dg .X......
                dg .X......
                dg .X......
                dg XXX.....
                dg ........

                dg ....X...
                dg ....X...
                dg ....X...
                dg ....X...
                dg ....X...
                dg X...X...
                dg .XXX....
                dg ........

                dg X...X...
                dg X..X....
                dg X.X.....
                dg XX......
                dg X.X.....
                dg X..X....
                dg X...X...
                dg ........

                dg X.......
                dg X.......
                dg X.......
                dg X.......
                dg X.......
                dg X.......
                dg XXXXX...
                dg ........

                dg X...X...
                dg XX.XX...
                dg X.X.X...
                dg X.X.X...
                dg X...X...
                dg X...X...
                dg X...X...
                dg ........

                dg X...X...
                dg X...X...
                dg XX..X...
                dg X.X.X...
                dg X..XX...
                dg X...X...
                dg X...X...
                dg ........

                dg .XXX....
                dg X...X...
                dg X...X...
                dg X...X...
                dg X...X...
                dg X...X...
                dg .XXX....
                dg ........

                dg XXXX....
                dg X...X...
                dg X...X...
                dg XXXX....
                dg X.......
                dg X.......
                dg X.......
                dg ........

                dg .XXX....
                dg X...X...
                dg X...X...
                dg X...X...
                dg X.X.X...
                dg X..X....
                dg .XX.X...
                dg ........

                dg XXXX....
                dg X...X...
                dg X...X...
                dg XXXX....
                dg X.X.....
                dg X..X....
                dg X...X...
                dg ........

                dg .XXX....
                dg X...X...
                dg X.......
                dg .XXX....
                dg ....X...
                dg X...X...
                dg .XXX....
                dg ........

                dg XXXXX...
                dg ..X.....
                dg ..X.....
                dg ..X.....
                dg ..X.....
                dg ..X.....
                dg ..X.....
                dg ........

                dg X...X...
                dg X...X...
                dg X...X...
                dg X...X...
                dg X...X...
                dg X...X...
                dg .XXX....
                dg ........

                dg X...X...
                dg X...X...
                dg X...X...
                dg X...X...
                dg X...X...
                dg .X.X....
                dg ..X.....
                dg ........

                dg X...X...
                dg X...X...
                dg X...X...
                dg X.X.X...
                dg X.X.X...
                dg XX.XX...
                dg X...X...
                dg ........

                dg X...X...
                dg X...X...
                dg .X.X....
                dg ..X.....
                dg .X.X....
                dg X...X...
                dg X...X...
                dg ........

                dg X...X...
                dg X...X...
                dg .X.X....
                dg ..X.....
                dg ..X.....
                dg ..X.....
                dg ..X.....
                dg ........

                dg XXXXX...
                dg ....X...
                dg ...X....
                dg ..X.....
                dg .X......
                dg X.......
                dg XXXXX...
                dg ........

                dg XXXXX...
                dg XX......
                dg XX......
                dg XX......
                dg XX......
                dg XX......
                dg XXXXX...
                dg ........

                dg ........
                dg X.......
                dg .X......
                dg ..X.....
                dg ...X....
                dg ....X...
                dg ........
                dg ........

                dg XXXXX...
                dg ...XX...
                dg ...XX...
                dg ...XX...
                dg ...XX...
                dg ...XX...
                dg XXXXX...
                dg ........

                dg ..X.....
                dg .XXX....
                dg X.X.X...
                dg ..X.....
                dg ..X.....
                dg ..X.....
                dg ........
                dg ........

                dg ........
                dg ........
                dg ........
                dg ........
                dg ........
                dg ........
                dg XXXXX...
                dg ........

                dg ........
                dg ........
                dg ........
                dg ........
                dg ........
                dg ........
                dg ........
                dg ........

                dg ........
                dg ........
                dg .XX.X...
                dg X..XX...
                dg X...X...
                dg X..XX...
                dg .XX.X...
                dg ........

                dg X.......
                dg X.......
                dg X.XX....
                dg XX..X...
                dg X...X...
                dg XX..X...
                dg X.XX....
                dg ........

                dg ........
                dg ........
                dg .XXXX...
                dg X.......
                dg X.......
                dg X.......
                dg .XXXX...
                dg ........

                dg ....X...
                dg ....X...
                dg .XX.X...
                dg X..XX...
                dg X...X...
                dg X..XX...
                dg .XX.X...
                dg ........

                dg ........
                dg ........
                dg .XXX....
                dg X...X...
                dg XXXXX...
                dg X.......
                dg .XXX....
                dg ........

                dg ..X.....
                dg .X......
                dg .X......
                dg XXX.....
                dg .X......
                dg .X......
                dg .X......
                dg ........

                dg ........
                dg ........
                dg .XXXX...
                dg X...X...
                dg X..XX...
                dg .XX.X...
                dg ....X...
                dg .XXX....

                dg X.......
                dg X.......
                dg XXXX....
                dg X...X...
                dg X...X...
                dg X...X...
                dg X...X...
                dg ........

                dg X.......
                dg ........
                dg X.......
                dg X.......
                dg X.......
                dg X.......
                dg X.......
                dg ........

                dg ........
                dg ..X.....
                dg ........
                dg ..X.....
                dg ..X.....
                dg ..X.....
                dg ..X.....
                dg XX......

                dg X.......
                dg X.......
                dg X..X....
                dg X.X.....
                dg XXX.....
                dg X..X....
                dg X...X...
                dg ........

                dg XX......
                dg .X......
                dg .X......
                dg .X......
                dg .X......
                dg .X......
                dg XXX.....
                dg ........

                dg ........
                dg ........
                dg XX.X....
                dg X.X.X...
                dg X.X.X...
                dg X.X.X...
                dg X.X.X...
                dg ........

                dg ........
                dg ........
                dg XXXX....
                dg X...X...
                dg X...X...
                dg X...X...
                dg X...X...
                dg ........

                dg ........
                dg ........
                dg .XXX....
                dg X...X...
                dg X...X...
                dg X...X...
                dg .XXX....
                dg ........

                dg ........
                dg ........
                dg XXXX....
                dg X...X...
                dg X...X...
                dg XXXX....
                dg X.......
                dg X.......

                dg ........
                dg ........
                dg .XXXX...
                dg X...X...
                dg X...X...
                dg .XXXX...
                dg ....X...
                dg ....X...

                dg ........
                dg ........
                dg X.XX....
                dg XX......
                dg X.......
                dg X.......
                dg X.......
                dg ........

                dg ........
                dg ........
                dg .XXXX...
                dg X.......
                dg .XXX....
                dg ....X...
                dg XXXX....
                dg ........

                dg ........
                dg ..X.....
                dg XXXXX...
                dg ..X.....
                dg ..X.....
                dg ..X.....
                dg ..x..... ; This used to be ...XX...
                dg ........

                dg ........
                dg ........
                dg X...X...
                dg X...X...
                dg X...X...
                dg X...X...
                dg .XXXX...
                dg ........

                dg ........
                dg ........
                dg X...X...
                dg X...X...
                dg .X.X....
                dg .X.X....
                dg ..X.....
                dg ........

                dg ........
                dg ........
                dg X...X...
                dg X...X...
                dg X...X...
                dg X.X.X...
                dg .X.X....
                dg ........

                dg ........
                dg ........
                dg X...X...
                dg .X.X....
                dg ..X.....
                dg .X.X....
                dg X...X...
                dg ........

                dg ........
                dg ........
                dg X..X....
                dg X..X....
                dg X..X....
                dg XXXX....
                dg ...X....
                dg XXX.....

                dg ........
                dg ........
                dg XXXXX...
                dg ...X....
                dg ..X.....
                dg .X......
                dg XXXXX...
                dg ........

                dg ..XX....
                dg .X......
                dg .X......
                dg X.......
                dg .X......
                dg .X......
                dg ..XX....
                dg ........

                dg ........
                dg X.......
                dg X.......
                dg ........
                dg ........
                dg X.......
                dg X.......
                dg ........

                dg XX......
                dg ..X.....
                dg ..X.....
                dg ...X....
                dg ..X.....
                dg ..X.....
                dg XX......
                dg ........

                dg .X.X....
                dg ........
                dg .XXX....
                dg X...X...
                dg XXXXX...
                dg X.......
                dg .XXX....
                dg ........

                dg ........
                dg ........
                dg ....X...
                dg .XXX....
                dg X.......
                dg ........
                dg ........
                dg ........

; Character widths

PrintWidths     defb $07,$09,$09,$09,$09,$09,$09,$03
                defb $05,$05,$05,$05,$03,$05,$05,$05
                defb $05,$03,$05,$05,$05,$05,$09,$09 ; " " ..
                defb $07,$09,$07,$02,$05,$09,$07,$09
                defb $06,$02,$04,$06,$06,$06,$06,$02
                defb $04,$04,$06,$06,$03,$06,$02,$06
                defb $06,$06,$06,$06,$06,$06,$06,$06
                defb $06,$06,$02,$03,$05,$06,$05,$06
                defb $06,$06,$06,$06,$06,$06,$06,$06
                defb $06,$04,$06,$06,$06,$06,$06,$06
                defb $06,$06,$06,$06,$06,$06,$06,$06
                defb $06,$06,$06,$06,$06,$06,$06,$06
                defb $06,$06,$06,$06,$06,$06,$04,$06
                defb $06,$02,$04,$06,$04,$06,$06,$06
                defb $06,$06,$05,$06,$06,$06,$06,$06
                defb $06,$05,$06,$05,$02,$05,$06,$06
                defb $03,$06,$0C,$12,$18,$1E,$24,$2A  ; Wide spaces $80..?
                defb $30,$36,$3C,$42,$48,$06,$06,$06
                defb $01,$02,$03,$04,$05,$06,$06,$06
                defb $06,$06,$06,$06,$06,$06,$06,$06
                defb $06,$06,$06,$06,$06,$06,$06,$06
                defb $06,$06,$06,$06,$06,$06,$06,$06
                defb $06,$06,$06,$06,$06,$06,$06,$06
                defb $06,$06,$06,$06,$06,$06,$06,$06
                defb $06,$06,$06,$06,$06,$06,$06,$06
                defb $06,$06,$06,$06,$06,$06,$06,$06
                defb $06,$06,$06,$06,$06,$06,$06,$06
                defb $06,$06,$06,$06,$06,$06,$06,$06
                defb $06,$06,$06,$06,$06,$06,$06,$06
                defb $06,$06,$06,$06,$06,$06,$06,$06
                defb $06,$06,$06,$06,$06,$06,$06,$06
                defb $06,$06,$06,$06,$06,$06,$06,$06

; Variables

pCurX           db 0            ; A logical X position
ppCurY          db 0            ; A logical Y position
pCurY           dw 0            ; The actual address
CurCol          db 0            ; The cursor colour

; This will overwrite the binary we're conmparing with
;                output_bin "test_250.bin",$8000,$1000   ; Generate binary

